#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wireless_controller_wids_profile
short_description: Configure wireless intrusion detection system (WIDS) profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify wireless_controller feature and wids_profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    wireless_controller_wids_profile:
        description:
            - Configure wireless intrusion detection system (WIDS) profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            ap-auto-suppress:
                description:
                    - Enable/disable on-wire rogue AP auto-suppression (default = disable).
                choices:
                    - enable
                    - disable
            ap-bgscan-disable-day:
                description:
                    - Optionally turn off scanning for one or more days of the week. Separate the days with a space. By default, no days are set.
                choices:
                    - sunday
                    - monday
                    - tuesday
                    - wednesday
                    - thursday
                    - friday
                    - saturday
            ap-bgscan-disable-end:
                description:
                    - "End time, using a 24-hour clock in the format of hh:mm, for disabling background scanning (default = 00:00)."
            ap-bgscan-disable-start:
                description:
                    - "Start time, using a 24-hour clock in the format of hh:mm, for disabling background scanning (default = 00:00)."
            ap-bgscan-duration:
                description:
                    - Listening time on a scanning channel (10 - 1000 msec, default = 20).
            ap-bgscan-idle:
                description:
                    - Waiting time for channel inactivity before scanning this channel (0 - 1000 msec, default = 0).
            ap-bgscan-intv:
                description:
                    - Period of time between scanning two channels (1 - 600 sec, default = 1).
            ap-bgscan-period:
                description:
                    - Period of time between background scans (60 - 3600 sec, default = 600).
            ap-bgscan-report-intv:
                description:
                    - Period of time between background scan reports (15 - 600 sec, default = 30).
            ap-fgscan-report-intv:
                description:
                    - Period of time between foreground scan reports (15 - 600 sec, default = 15).
            ap-scan:
                description:
                    - Enable/disable rogue AP detection.
                choices:
                    - disable
                    - enable
            ap-scan-passive:
                description:
                    - Enable/disable passive scanning. Enable means do not send probe request on any channels (default = disable).
                choices:
                    - enable
                    - disable
            asleap-attack:
                description:
                    - Enable/disable asleap attack detection (default = disable).
                choices:
                    - enable
                    - disable
            assoc-flood-thresh:
                description:
                    - The threshold value for association frame flooding.
            assoc-flood-time:
                description:
                    - Number of seconds after which a station is considered not connected.
            assoc-frame-flood:
                description:
                    - Enable/disable association frame flooding detection (default = disable).
                choices:
                    - enable
                    - disable
            auth-flood-thresh:
                description:
                    - The threshold value for authentication frame flooding.
            auth-flood-time:
                description:
                    - Number of seconds after which a station is considered not connected.
            auth-frame-flood:
                description:
                    - Enable/disable authentication frame flooding detection (default = disable).
                choices:
                    - enable
                    - disable
            comment:
                description:
                    - Comment.
            deauth-broadcast:
                description:
                    - Enable/disable broadcasting de-authentication detection (default = disable).
                choices:
                    - enable
                    - disable
            deauth-unknown-src-thresh:
                description:
                    - "Threshold value per second to deauth unknown src for DoS attack (0: no limit)."
            eapol-fail-flood:
                description:
                    - Enable/disable EAPOL-Failure flooding (to AP) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-fail-intv:
                description:
                    - The detection interval for EAPOL-Failure flooding (1 - 3600 sec).
            eapol-fail-thresh:
                description:
                    - The threshold value for EAPOL-Failure flooding in specified interval.
            eapol-logoff-flood:
                description:
                    - Enable/disable EAPOL-Logoff flooding (to AP) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-logoff-intv:
                description:
                    - The detection interval for EAPOL-Logoff flooding (1 - 3600 sec).
            eapol-logoff-thresh:
                description:
                    - The threshold value for EAPOL-Logoff flooding in specified interval.
            eapol-pre-fail-flood:
                description:
                    - Enable/disable premature EAPOL-Failure flooding (to STA) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-pre-fail-intv:
                description:
                    - The detection interval for premature EAPOL-Failure flooding (1 - 3600 sec).
            eapol-pre-fail-thresh:
                description:
                    - The threshold value for premature EAPOL-Failure flooding in specified interval.
            eapol-pre-succ-flood:
                description:
                    - Enable/disable premature EAPOL-Success flooding (to STA) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-pre-succ-intv:
                description:
                    - The detection interval for premature EAPOL-Success flooding (1 - 3600 sec).
            eapol-pre-succ-thresh:
                description:
                    - The threshold value for premature EAPOL-Success flooding in specified interval.
            eapol-start-flood:
                description:
                    - Enable/disable EAPOL-Start flooding (to AP) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-start-intv:
                description:
                    - The detection interval for EAPOL-Start flooding (1 - 3600 sec).
            eapol-start-thresh:
                description:
                    - The threshold value for EAPOL-Start flooding in specified interval.
            eapol-succ-flood:
                description:
                    - Enable/disable EAPOL-Success flooding (to AP) detection (default = disable).
                choices:
                    - enable
                    - disable
            eapol-succ-intv:
                description:
                    - The detection interval for EAPOL-Success flooding (1 - 3600 sec).
            eapol-succ-thresh:
                description:
                    - The threshold value for EAPOL-Success flooding in specified interval.
            invalid-mac-oui:
                description:
                    - Enable/disable invalid MAC OUI detection.
                choices:
                    - enable
                    - disable
            long-duration-attack:
                description:
                    - Enable/disable long duration attack detection based on user configured threshold (default = disable).
                choices:
                    - enable
                    - disable
            long-duration-thresh:
                description:
                    - Threshold value for long duration attack detection (1000 - 32767 usec, default = 8200).
            name:
                description:
                    - WIDS profile name.
                required: true
            null-ssid-probe-resp:
                description:
                    - Enable/disable null SSID probe response detection (default = disable).
                choices:
                    - enable
                    - disable
            sensor-mode:
                description:
                    - Scan WiFi nearby stations (default = disable).
                choices:
                    - disable
                    - foreign
                    - both
            spoofed-deauth:
                description:
                    - Enable/disable spoofed de-authentication attack detection (default = disable).
                choices:
                    - enable
                    - disable
            weak-wep-iv:
                description:
                    - Enable/disable weak WEP IV (Initialization Vector) detection (default = disable).
                choices:
                    - enable
                    - disable
            wireless-bridge:
                description:
                    - Enable/disable wireless bridge detection (default = disable).
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure wireless intrusion detection system (WIDS) profiles.
    fortios_wireless_controller_wids_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wireless_controller_wids_profile:
        state: "present"
        ap-auto-suppress: "enable"
        ap-bgscan-disable-day: "sunday"
        ap-bgscan-disable-end: "<your_own_value>"
        ap-bgscan-disable-start: "<your_own_value>"
        ap-bgscan-duration: "7"
        ap-bgscan-idle: "8"
        ap-bgscan-intv: "9"
        ap-bgscan-period: "10"
        ap-bgscan-report-intv: "11"
        ap-fgscan-report-intv: "12"
        ap-scan: "disable"
        ap-scan-passive: "enable"
        asleap-attack: "enable"
        assoc-flood-thresh: "16"
        assoc-flood-time: "17"
        assoc-frame-flood: "enable"
        auth-flood-thresh: "19"
        auth-flood-time: "20"
        auth-frame-flood: "enable"
        comment: "Comment."
        deauth-broadcast: "enable"
        deauth-unknown-src-thresh: "24"
        eapol-fail-flood: "enable"
        eapol-fail-intv: "26"
        eapol-fail-thresh: "27"
        eapol-logoff-flood: "enable"
        eapol-logoff-intv: "29"
        eapol-logoff-thresh: "30"
        eapol-pre-fail-flood: "enable"
        eapol-pre-fail-intv: "32"
        eapol-pre-fail-thresh: "33"
        eapol-pre-succ-flood: "enable"
        eapol-pre-succ-intv: "35"
        eapol-pre-succ-thresh: "36"
        eapol-start-flood: "enable"
        eapol-start-intv: "38"
        eapol-start-thresh: "39"
        eapol-succ-flood: "enable"
        eapol-succ-intv: "41"
        eapol-succ-thresh: "42"
        invalid-mac-oui: "enable"
        long-duration-attack: "enable"
        long-duration-thresh: "45"
        name: "default_name_46"
        null-ssid-probe-resp: "enable"
        sensor-mode: "disable"
        spoofed-deauth: "enable"
        weak-wep-iv: "enable"
        wireless-bridge: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_wireless_controller_wids_profile_data(json):
    option_list = ['ap-auto-suppress', 'ap-bgscan-disable-day', 'ap-bgscan-disable-end',
                   'ap-bgscan-disable-start', 'ap-bgscan-duration', 'ap-bgscan-idle',
                   'ap-bgscan-intv', 'ap-bgscan-period', 'ap-bgscan-report-intv',
                   'ap-fgscan-report-intv', 'ap-scan', 'ap-scan-passive',
                   'asleap-attack', 'assoc-flood-thresh', 'assoc-flood-time',
                   'assoc-frame-flood', 'auth-flood-thresh', 'auth-flood-time',
                   'auth-frame-flood', 'comment', 'deauth-broadcast',
                   'deauth-unknown-src-thresh', 'eapol-fail-flood', 'eapol-fail-intv',
                   'eapol-fail-thresh', 'eapol-logoff-flood', 'eapol-logoff-intv',
                   'eapol-logoff-thresh', 'eapol-pre-fail-flood', 'eapol-pre-fail-intv',
                   'eapol-pre-fail-thresh', 'eapol-pre-succ-flood', 'eapol-pre-succ-intv',
                   'eapol-pre-succ-thresh', 'eapol-start-flood', 'eapol-start-intv',
                   'eapol-start-thresh', 'eapol-succ-flood', 'eapol-succ-intv',
                   'eapol-succ-thresh', 'invalid-mac-oui', 'long-duration-attack',
                   'long-duration-thresh', 'name', 'null-ssid-probe-resp',
                   'sensor-mode', 'spoofed-deauth', 'weak-wep-iv',
                   'wireless-bridge']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def wireless_controller_wids_profile(data, fos):
    vdom = data['vdom']
    wireless_controller_wids_profile_data = data['wireless_controller_wids_profile']
    flattened_data = flatten_multilists_attributes(wireless_controller_wids_profile_data)
    filtered_data = filter_wireless_controller_wids_profile_data(flattened_data)
    if wireless_controller_wids_profile_data['state'] == "present":
        return fos.set('wireless-controller',
                       'wids-profile',
                       data=filtered_data,
                       vdom=vdom)

    elif wireless_controller_wids_profile_data['state'] == "absent":
        return fos.delete('wireless-controller',
                          'wids-profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_wireless_controller(data, fos):
    login(data, fos)

    if data['wireless_controller_wids_profile']:
        resp = wireless_controller_wids_profile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "wireless_controller_wids_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "ap-auto-suppress": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "ap-bgscan-disable-day": {"required": False, "type": "str",
                                          "choices": ["sunday", "monday", "tuesday",
                                                      "wednesday", "thursday", "friday",
                                                      "saturday"]},
                "ap-bgscan-disable-end": {"required": False, "type": "str"},
                "ap-bgscan-disable-start": {"required": False, "type": "str"},
                "ap-bgscan-duration": {"required": False, "type": "int"},
                "ap-bgscan-idle": {"required": False, "type": "int"},
                "ap-bgscan-intv": {"required": False, "type": "int"},
                "ap-bgscan-period": {"required": False, "type": "int"},
                "ap-bgscan-report-intv": {"required": False, "type": "int"},
                "ap-fgscan-report-intv": {"required": False, "type": "int"},
                "ap-scan": {"required": False, "type": "str",
                            "choices": ["disable", "enable"]},
                "ap-scan-passive": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "asleap-attack": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "assoc-flood-thresh": {"required": False, "type": "int"},
                "assoc-flood-time": {"required": False, "type": "int"},
                "assoc-frame-flood": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "auth-flood-thresh": {"required": False, "type": "int"},
                "auth-flood-time": {"required": False, "type": "int"},
                "auth-frame-flood": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "comment": {"required": False, "type": "str"},
                "deauth-broadcast": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "deauth-unknown-src-thresh": {"required": False, "type": "int"},
                "eapol-fail-flood": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "eapol-fail-intv": {"required": False, "type": "int"},
                "eapol-fail-thresh": {"required": False, "type": "int"},
                "eapol-logoff-flood": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "eapol-logoff-intv": {"required": False, "type": "int"},
                "eapol-logoff-thresh": {"required": False, "type": "int"},
                "eapol-pre-fail-flood": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "eapol-pre-fail-intv": {"required": False, "type": "int"},
                "eapol-pre-fail-thresh": {"required": False, "type": "int"},
                "eapol-pre-succ-flood": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "eapol-pre-succ-intv": {"required": False, "type": "int"},
                "eapol-pre-succ-thresh": {"required": False, "type": "int"},
                "eapol-start-flood": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "eapol-start-intv": {"required": False, "type": "int"},
                "eapol-start-thresh": {"required": False, "type": "int"},
                "eapol-succ-flood": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "eapol-succ-intv": {"required": False, "type": "int"},
                "eapol-succ-thresh": {"required": False, "type": "int"},
                "invalid-mac-oui": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "long-duration-attack": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "long-duration-thresh": {"required": False, "type": "int"},
                "name": {"required": True, "type": "str"},
                "null-ssid-probe-resp": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "sensor-mode": {"required": False, "type": "str",
                                "choices": ["disable", "foreign", "both"]},
                "spoofed-deauth": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "weak-wep-iv": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "wireless-bridge": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_wireless_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_autosupport \
    import NetAppONTAPasup as asup_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.kind = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'asup':
            xml = self.build_asup_config_info(self.params)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_asup_config_info(asup_data):
        ''' build xml data for asup-config '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {'attributes': {'autosupport-config-info': {
            'node-name': asup_data['node_name'],
            'is-enabled': asup_data['is_enabled'],
            'is-support-enabled': asup_data['support'],
            'proxy-url': asup_data['proxy_url'],
            'post-url': asup_data['post_url'],
            'transport': asup_data['transport'],
            'is-node-in-subject': 'false',
            'from': 'test',
            'mail-hosts': [{'string': '1.2.3.4'}, {'string': '4.5.6.8'}],
            'noteto': [{'mail-address': 'abc@test.com'},
                       {'mail-address': 'def@test.com'}],
        }}}
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.mock_asup = {
            'node_name': 'test-vsim1',
            'transport': 'https',
            'support': 'false',
            'post_url': 'testbed.netapp.com/asupprod/post/1.0/postAsup',
            'proxy_url': 'something.com',
        }

    def mock_args(self):
        return {
            'node_name': self.mock_asup['node_name'],
            'transport': self.mock_asup['transport'],
            'support': self.mock_asup['support'],
            'post_url': self.mock_asup['post_url'],
            'proxy_url': self.mock_asup['proxy_url'],
            'hostname': 'host',
            'username': 'admin',
            'password': 'password',
        }

    def get_asup_mock_object(self, kind=None, enabled='false'):
        """
        Helper method to return an na_ontap_volume object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_volume object
        """
        asup_obj = asup_module()
        asup_obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            asup_obj.server = MockONTAPConnection()
        else:
            data = self.mock_asup
            data['is_enabled'] = enabled
            asup_obj.server = MockONTAPConnection(kind='asup', data=data)
        return asup_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            asup_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_enable_asup(self):
        ''' a more interesting test '''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object('asup').apply()
        assert exc.value.args[0]['changed']

    def test_disable_asup(self):
        ''' a more interesting test '''
        # enable asup
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object(kind='asup', enabled='true').apply()
        assert exc.value.args[0]['changed']

    def test_result_from_get(self):
        ''' Check boolean and service_state conversion from get '''
        data = self.mock_args()
        set_module_args(data)
        obj = self.get_asup_mock_object(kind='asup', enabled='true')
        # constructed based on valued passed in self.mock_asup and build_asup_config_info()
        expected_dict = {
            'node_name': 'test-vsim1',
            'service_state': 'started',
            'support': False,
            'hostname_in_subject': False,
            'transport': self.mock_asup['transport'],
            'post_url': self.mock_asup['post_url'],
            'proxy_url': self.mock_asup['proxy_url'],
            'from_address': 'test',
            'mail_hosts': ['1.2.3.4', '4.5.6.8'],
            'partner_addresses': [],
            'to_addresses': [],
            'noteto': ['abc@test.com', 'def@test.com']
        }
        result = obj.get_autosupport_config()
        assert result == expected_dict

    def test_modify_config(self):
        ''' Check boolean and service_state conversion from get '''
        data = self.mock_args()
        data['transport'] = 'http'
        data['post_url'] = 'somethingelse.com'
        data['proxy_url'] = 'somethingelse.com'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object('asup').apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_autosupport.NetAppONTAPasup.get_autosupport_config')
    def test_get_called(self, get_asup):
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object('asup').apply()
        get_asup.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_autosupport.NetAppONTAPasup.modify_autosupport_config')
    def test_modify_called(self, modify_asup):
        data = self.mock_args()
        data['transport'] = 'http'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object('asup').apply()
        modify_asup.assert_called_with({'transport': 'http', 'service_state': 'started'})

    @patch('ansible.modules.storage.netapp.na_ontap_autosupport.NetAppONTAPasup.modify_autosupport_config')
    @patch('ansible.modules.storage.netapp.na_ontap_autosupport.NetAppONTAPasup.get_autosupport_config')
    def test_modify_not_called(self, get_asup, modify_asup):
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_asup_mock_object('asup').apply()
        get_asup.assert_called_with()
        modify_asup.assert_not_called()

    def test_modify_packet(self):
        '''check XML construction for nested attributes like mail-hosts, noteto, partner-address, and to'''
        data = self.mock_args()
        set_module_args(data)
        obj = self.get_asup_mock_object(kind='asup', enabled='true')
        modify_dict = {
            'noteto': ['one@test.com'],
            'partner_addresses': ['firstpartner@test.com'],
            'mail_hosts': ['1.1.1.1'],
            'to_addresses': ['first@test.com']
        }
        obj.modify_autosupport_config(modify_dict)
        xml = obj.server.xml_in
        for key in ['noteto', 'to', 'partner-address']:
            assert xml[key] is not None
            assert xml[key]['mail-address'] is not None
        assert xml['noteto']['mail-address'] == modify_dict['noteto'][0]
        assert xml['to']['mail-address'] == modify_dict['to_addresses'][0]
        assert xml['partner-address']['mail-address'] == modify_dict['partner_addresses'][0]
        assert xml['mail-hosts'] is not None
        assert xml['mail-hosts']['string'] is not None
        assert xml['mail-hosts']['string'] == modify_dict['mail_hosts'][0]



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2019, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
import pytest

from ansible.modules.cloud.misc import virt_net

from units.compat import mock


virt_net.libvirt = None
virt_net.HAS_VIRT = True


class DummyNetwork():
    def __init__(self, name, isActive=True):
        self._name = name
        self._isActive = isActive

    def name(self):
        return self._name

    def isActive(self):
        return self._isActive


class DummyLibvirtConn():
    def __init__(self):
        self._network = [
            DummyNetwork("inactive_net", isActive=False),
            DummyNetwork("active_net", isActive=True)]

    def listNetworks(self):
        return [i.name() for i in self._network]

    def networkLookupByName(self, name):
        for i in self._network:
            if i.name() == name:
                return i

    def listDefinedNetworks(self):
        return []


class DummyLibvirt():
    VIR_ERR_NETWORK_EXIST = 'VIR_ERR_NETWORK_EXIST'

    @classmethod
    def open(cls, uri):
        return DummyLibvirtConn()

    class libvirtError(Exception):
        def __init__(self):
            self.error_code

        def get_error_code(self):
            return self.error_code


@pytest.fixture
def dummy_libvirt(monkeypatch):
    monkeypatch.setattr(virt_net, 'libvirt', DummyLibvirt)
    return DummyLibvirt


@pytest.fixture
def virt_net_obj(dummy_libvirt):
    return virt_net.VirtNetwork('qemu:///nowhere', mock.MagicMock())



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Hetzner Cloud GmbH <info@hetzner-cloud.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: hcloud_server_facts

short_description: Gather facts about your Hetzner Cloud servers.

version_added: "2.8"

description:
    - Gather facts about your Hetzner Cloud servers.

author:
    - Lukas Kaemmerling (@lkaemmerling)

options:
    id:
        description:
            - The ID of the server you want to get.
        type: int
    name:
        description:
            - The name of the server you want to get.
        type: str
    label_selector:
        description:
            - The label selector for the servery you want to get.
        type: str
extends_documentation_fragment: hcloud
"""

EXAMPLES = """
- name: Gather hcloud server facts
  local_action:
    module: hcloud_server_facts

- name: Print the gathered facts
  debug:
    var: ansible_facts.hcloud_server_facts
"""

RETURN = """
hcloud_server_facts:
    description: The server facts as list
    returned: always
    type: complex
    contains:
        id:
            description: Numeric identifier of the server
            returned: always
            type: int
            sample: 1937415
        name:
            description: Name of the server
            returned: always
            type: str
            sample: my-server
        status:
            description: Status of the server
            returned: always
            type: str
            sample: running
        server_type:
            description: Name of the server type of the server
            returned: always
            type: str
            sample: cx11
        ipv4_address:
            description: Public IPv4 address of the server
            returned: always
            type: str
            sample: 116.203.104.109
        ipv6:
            description: IPv6 network of the server
            returned: always
            type: str
            sample: 2a01:4f8:1c1c:c140::/64
        location:
            description: Name of the location of the server
            returned: always
            type: str
            sample: fsn1
        datacenter:
            description: Name of the datacenter of the server
            returned: always
            type: str
            sample: fsn1-dc14
        rescue_enabled:
            description: True if rescue mode is enabled, Server will then boot into rescue system on next reboot
            returned: always
            type: bool
            sample: false
        backup_window:
            description: Time window (UTC) in which the backup will run, or null if the backups are not enabled
            returned: always
            type: bool
            sample: 22-02
        labels:
            description: User-defined labels (key-value pairs)
            returned: always
            type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
from ansible.module_utils.hcloud import Hcloud

try:
    from hcloud import APIException
except ImportError:
    pass


class AnsibleHcloudServerFacts(Hcloud):
    def __init__(self, module):
        Hcloud.__init__(self, module, "hcloud_server_facts")
        self.hcloud_server_facts = None

    def _prepare_result(self):
        tmp = []

        for server in self.hcloud_server_facts:
            if server is not None:
                tmp.append({
                    "id": to_native(server.id),
                    "name": to_native(server.name),
                    "ipv4_address": to_native(server.public_net.ipv4.ip),
                    "ipv6": to_native(server.public_net.ipv6.ip),
                    "image": to_native(server.image.name),
                    "server_type": to_native(server.server_type.name),
                    "datacenter": to_native(server.datacenter.name),
                    "location": to_native(server.datacenter.location.name),
                    "rescue_enabled": server.rescue_enabled,
                    "backup_window": to_native(server.backup_window),
                    "labels": server.labels,
                    "status": to_native(server.status),
                })
        return tmp

    def get_servers(self):
        try:
            if self.module.params.get("id") is not None:
                self.hcloud_server_facts = [self.client.servers.get_by_id(
                    self.module.params.get("id")
                )]
            elif self.module.params.get("name") is not None:
                self.hcloud_server_facts = [self.client.servers.get_by_name(
                    self.module.params.get("name")
                )]
            elif self.module.params.get("label_selector") is not None:
                self.hcloud_server_facts = self.client.servers.get_all(
                    label_selector=self.module.params.get("label_selector"))
            else:
                self.hcloud_server_facts = self.client.servers.get_all()

        except APIException as e:
            self.module.fail_json(msg=e.message)

    @staticmethod
    def define_module():
        return AnsibleModule(
            argument_spec=dict(
                id={"type": "int"},
                name={"type": "str"},
                label_selector={"type": "str"},
                **Hcloud.base_module_arguments()
            ),
            supports_check_mode=True,
        )


def main():
    module = AnsibleHcloudServerFacts.define_module()

    hcloud = AnsibleHcloudServerFacts(module)
    hcloud.get_servers()
    result = hcloud.get_result()
    ansible_facts = {
        'hcloud_server_facts': result['hcloud_server_facts']
    }
    module.exit_json(ansible_facts=ansible_facts)


if __name__ == "__main__":
    main()



import json
import pytest

from units.compat.mock import patch, MagicMock

from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
from ansible.module_utils.k8s.common import K8sAnsibleMixin
from ansible.module_utils.k8s.raw import KubernetesRawModule
from ansible.module_utils.kubevirt import KubeVirtRawModule

from ansible.modules.cloud.kubevirt import kubevirt_vm as mymodule

openshiftdynamic = pytest.importorskip("openshift.dynamic")
helpexceptions = pytest.importorskip("openshift.helper.exceptions")

KIND = 'VirtulMachine'
RESOURCE_DEFAULT_ARGS = {'api_version': 'v1alpha3', 'group': 'kubevirt.io',
                         'prefix': 'apis', 'namespaced': True}


def set_module_args(args):
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught
    by the test case"""
    def __init__(self, **kwargs):
        for k in kwargs:
            setattr(self, k, kwargs[k])

    def __getitem__(self, attr):
        return getattr(self, attr)


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught
    by the test case"""
    def __init__(self, **kwargs):
        for k in kwargs:
            setattr(self, k, kwargs[k])

    def __getitem__(self, attr):
        return getattr(self, attr)


def exit_json(*args, **kwargs):
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(**kwargs)


def fail_json(*args, **kwargs):
    raise AnsibleFailJson(**kwargs)


@pytest.fixture(autouse=True)
def setup_mixtures(monkeypatch):
    monkeypatch.setattr(
        KubernetesRawModule, "exit_json", exit_json)
    monkeypatch.setattr(
        KubernetesRawModule, "fail_json", fail_json)
    # Create mock methods in Resource directly, otherwise dyn client
    # tries binding those to corresponding methods in DynamicClient
    # (with partial()), which is more problematic to intercept
    openshiftdynamic.Resource.get = MagicMock()
    openshiftdynamic.Resource.create = MagicMock()
    openshiftdynamic.Resource.delete = MagicMock()
    openshiftdynamic.Resource.patch = MagicMock()
    # Globally mock some methods, since all tests will use this
    K8sAnsibleMixin.get_api_client = MagicMock()
    K8sAnsibleMixin.get_api_client.return_value = None
    K8sAnsibleMixin.find_resource = MagicMock()
    KubeVirtRawModule.find_supported_resource = MagicMock()


def test_vm_multus_creation():
    # Desired state:
    args = dict(
        state='present', name='testvm',
        namespace='vms',
        interfaces=[
            {'bridge': {}, 'name': 'default', 'network': {'pod': {}}},
            {'bridge': {}, 'name': 'mynet', 'network': {'multus': {'networkName': 'mynet'}}},
        ],
        wait=False,
    )
    set_module_args(args)

    # State as "returned" by the "k8s cluster":
    openshiftdynamic.Resource.get.return_value = None
    resource_args = dict(kind=KIND, **RESOURCE_DEFAULT_ARGS)
    KubeVirtRawModule.find_supported_resource.return_value = openshiftdynamic.Resource(**resource_args)

    # Actual test:
    with pytest.raises(AnsibleExitJson) as result:
        mymodule.KubeVirtVM().execute_module()
    assert result.value['changed']
    assert result.value['method'] == 'create'


@pytest.mark.parametrize("_wait", (False, True))
def test_resource_absent(_wait):
    # Desired state:
    args = dict(
        state='absent', name='testvmi',
        namespace='vms',
        wait=_wait,
    )
    set_module_args(args)

    # State as "returned" by the "k8s cluster":
    openshiftdynamic.Resource.get.return_value = None
    resource_args = dict(kind=KIND, **RESOURCE_DEFAULT_ARGS)
    KubeVirtRawModule.find_supported_resource.return_value = openshiftdynamic.Resource(**resource_args)

    # Actual test:
    with pytest.raises(AnsibleExitJson) as result:
        mymodule.KubeVirtVM().execute_module()
    assert result.value['method'] == 'delete'
    assert not result.value['kubevirt_vm']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_dhcp_server
short_description: Configure DHCP servers in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify system_dhcp feature and server category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    system_dhcp_server:
        description:
            - Configure DHCP servers.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            auto-configuration:
                description:
                    - Enable/disable auto configuration.
                choices:
                    - disable
                    - enable
            conflicted-ip-timeout:
                description:
                    - Time in seconds to wait after a conflicted IP address is removed from the DHCP range before it can be reused.
            ddns-auth:
                description:
                    - DDNS authentication mode.
                choices:
                    - disable
                    - tsig
            ddns-key:
                description:
                    - DDNS update key (base 64 encoding).
            ddns-keyname:
                description:
                    - DDNS update key name.
            ddns-server-ip:
                description:
                    - DDNS server IP.
            ddns-ttl:
                description:
                    - TTL.
            ddns-update:
                description:
                    - Enable/disable DDNS update for DHCP.
                choices:
                    - disable
                    - enable
            ddns-update-override:
                description:
                    - Enable/disable DDNS update override for DHCP.
                choices:
                    - disable
                    - enable
            ddns-zone:
                description:
                    - Zone of your domain name (ex. DDNS.com).
            default-gateway:
                description:
                    - Default gateway IP address assigned by the DHCP server.
            dns-server1:
                description:
                    - DNS server 1.
            dns-server2:
                description:
                    - DNS server 2.
            dns-server3:
                description:
                    - DNS server 3.
            dns-service:
                description:
                    - Options for assigning DNS servers to DHCP clients.
                choices:
                    - local
                    - default
                    - specify
            domain:
                description:
                    - Domain name suffix for the IP addresses that the DHCP server assigns to clients.
            exclude-range:
                description:
                    - Exclude one or more ranges of IP addresses from being assigned to clients.
                suboptions:
                    end-ip:
                        description:
                            - End of IP range.
                    id:
                        description:
                            - ID.
                        required: true
                    start-ip:
                        description:
                            - Start of IP range.
            filename:
                description:
                    - Name of the boot file on the TFTP server.
            forticlient-on-net-status:
                description:
                    - Enable/disable FortiClient-On-Net service for this DHCP server.
                choices:
                    - disable
                    - enable
            id:
                description:
                    - ID.
                required: true
            interface:
                description:
                    - DHCP server can assign IP configurations to clients connected to this interface. Source system.interface.name.
            ip-mode:
                description:
                    - Method used to assign client IP.
                choices:
                    - range
                    - usrgrp
            ip-range:
                description:
                    - DHCP IP range configuration.
                suboptions:
                    end-ip:
                        description:
                            - End of IP range.
                    id:
                        description:
                            - ID.
                        required: true
                    start-ip:
                        description:
                            - Start of IP range.
            ipsec-lease-hold:
                description:
                    - DHCP over IPsec leases expire this many seconds after tunnel down (0 to disable forced-expiry).
            lease-time:
                description:
                    - Lease time in seconds, 0 means unlimited.
            mac-acl-default-action:
                description:
                    - MAC access control default action (allow or block assigning IP settings).
                choices:
                    - assign
                    - block
            netmask:
                description:
                    - Netmask assigned by the DHCP server.
            next-server:
                description:
                    - IP address of a server (for example, a TFTP sever) that DHCP clients can download a boot file from.
            ntp-server1:
                description:
                    - NTP server 1.
            ntp-server2:
                description:
                    - NTP server 2.
            ntp-server3:
                description:
                    - NTP server 3.
            ntp-service:
                description:
                    - Options for assigning Network Time Protocol (NTP) servers to DHCP clients.
                choices:
                    - local
                    - default
                    - specify
            options:
                description:
                    - DHCP options.
                suboptions:
                    code:
                        description:
                            - DHCP option code.
                    id:
                        description:
                            - ID.
                        required: true
                    ip:
                        description:
                            - DHCP option IPs.
                    type:
                        description:
                            - DHCP option type.
                        choices:
                            - hex
                            - string
                            - ip
                    value:
                        description:
                            - DHCP option value.
            reserved-address:
                description:
                    - Options for the DHCP server to assign IP settings to specific MAC addresses.
                suboptions:
                    action:
                        description:
                            - Options for the DHCP server to configure the client with the reserved MAC address.
                        choices:
                            - assign
                            - block
                            - reserved
                    description:
                        description:
                            - Description.
                    id:
                        description:
                            - ID.
                        required: true
                    ip:
                        description:
                            - IP address to be reserved for the MAC address.
                    mac:
                        description:
                            - MAC address of the client that will get the reserved IP address.
            server-type:
                description:
                    - DHCP server can be a normal DHCP server or an IPsec DHCP server.
                choices:
                    - regular
                    - ipsec
            status:
                description:
                    - Enable/disable this DHCP configuration.
                choices:
                    - disable
                    - enable
            tftp-server:
                description:
                    - One or more hostnames or IP addresses of the TFTP servers in quotes separated by spaces.
                suboptions:
                    tftp-server:
                        description:
                            - TFTP server.
                        required: true
            timezone:
                description:
                    - Select the time zone to be assigned to DHCP clients.
                choices:
                    - 01
                    - 02
                    - 03
                    - 04
                    - 05
                    - 81
                    - 06
                    - 07
                    - 08
                    - 09
                    - 10
                    - 11
                    - 12
                    - 13
                    - 74
                    - 14
                    - 77
                    - 15
                    - 87
                    - 16
                    - 17
                    - 18
                    - 19
                    - 20
                    - 75
                    - 21
                    - 22
                    - 23
                    - 24
                    - 80
                    - 79
                    - 25
                    - 26
                    - 27
                    - 28
                    - 78
                    - 29
                    - 30
                    - 31
                    - 32
                    - 33
                    - 34
                    - 35
                    - 36
                    - 37
                    - 38
                    - 83
                    - 84
                    - 40
                    - 85
                    - 41
                    - 42
                    - 43
                    - 39
                    - 44
                    - 46
                    - 47
                    - 51
                    - 48
                    - 45
                    - 49
                    - 50
                    - 52
                    - 53
                    - 54
                    - 55
                    - 56
                    - 57
                    - 58
                    - 59
                    - 60
                    - 62
                    - 63
                    - 61
                    - 64
                    - 65
                    - 66
                    - 67
                    - 68
                    - 69
                    - 70
                    - 71
                    - 72
                    - 00
                    - 82
                    - 73
                    - 86
                    - 76
            timezone-option:
                description:
                    - Options for the DHCP server to set the client's time zone.
                choices:
                    - disable
                    - default
                    - specify
            vci-match:
                description:
                    - Enable/disable vendor class identifier (VCI) matching. When enabled only DHCP requests with a matching VCI are served.
                choices:
                    - disable
                    - enable
            vci-string:
                description:
                    - One or more VCI strings in quotes separated by spaces.
                suboptions:
                    vci-string:
                        description:
                            - VCI strings.
                        required: true
            wifi-ac1:
                description:
                    - WiFi Access Controller 1 IP address (DHCP option 138, RFC 5417).
            wifi-ac2:
                description:
                    - WiFi Access Controller 2 IP address (DHCP option 138, RFC 5417).
            wifi-ac3:
                description:
                    - WiFi Access Controller 3 IP address (DHCP option 138, RFC 5417).
            wins-server1:
                description:
                    - WINS server 1.
            wins-server2:
                description:
                    - WINS server 2.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure DHCP servers.
    fortios_system_dhcp_server:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      system_dhcp_server:
        state: "present"
        auto-configuration: "disable"
        conflicted-ip-timeout: "4"
        ddns-auth: "disable"
        ddns-key: "<your_own_value>"
        ddns-keyname: "<your_own_value>"
        ddns-server-ip: "<your_own_value>"
        ddns-ttl: "9"
        ddns-update: "disable"
        ddns-update-override: "disable"
        ddns-zone: "<your_own_value>"
        default-gateway: "<your_own_value>"
        dns-server1: "<your_own_value>"
        dns-server2: "<your_own_value>"
        dns-server3: "<your_own_value>"
        dns-service: "local"
        domain: "<your_own_value>"
        exclude-range:
         -
            end-ip: "<your_own_value>"
            id:  "21"
            start-ip: "<your_own_value>"
        filename: "<your_own_value>"
        forticlient-on-net-status: "disable"
        id:  "25"
        interface: "<your_own_value> (source system.interface.name)"
        ip-mode: "range"
        ip-range:
         -
            end-ip: "<your_own_value>"
            id:  "30"
            start-ip: "<your_own_value>"
        ipsec-lease-hold: "32"
        lease-time: "33"
        mac-acl-default-action: "assign"
        netmask: "<your_own_value>"
        next-server: "<your_own_value>"
        ntp-server1: "<your_own_value>"
        ntp-server2: "<your_own_value>"
        ntp-server3: "<your_own_value>"
        ntp-service: "local"
        options:
         -
            code: "42"
            id:  "43"
            ip: "<your_own_value>"
            type: "hex"
            value: "<your_own_value>"
        reserved-address:
         -
            action: "assign"
            description: "<your_own_value>"
            id:  "50"
            ip: "<your_own_value>"
            mac: "<your_own_value>"
        server-type: "regular"
        status: "disable"
        tftp-server:
         -
            tftp-server: "<your_own_value>"
        timezone: "01"
        timezone-option: "disable"
        vci-match: "disable"
        vci-string:
         -
            vci-string: "<your_own_value>"
        wifi-ac1: "<your_own_value>"
        wifi-ac2: "<your_own_value>"
        wifi-ac3: "<your_own_value>"
        wins-server1: "<your_own_value>"
        wins-server2: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_dhcp_server_data(json):
    option_list = ['auto-configuration', 'conflicted-ip-timeout', 'ddns-auth',
                   'ddns-key', 'ddns-keyname', 'ddns-server-ip',
                   'ddns-ttl', 'ddns-update', 'ddns-update-override',
                   'ddns-zone', 'default-gateway', 'dns-server1',
                   'dns-server2', 'dns-server3', 'dns-service',
                   'domain', 'exclude-range', 'filename',
                   'forticlient-on-net-status', 'id', 'interface',
                   'ip-mode', 'ip-range', 'ipsec-lease-hold',
                   'lease-time', 'mac-acl-default-action', 'netmask',
                   'next-server', 'ntp-server1', 'ntp-server2',
                   'ntp-server3', 'ntp-service', 'options',
                   'reserved-address', 'server-type', 'status',
                   'tftp-server', 'timezone', 'timezone-option',
                   'vci-match', 'vci-string', 'wifi-ac1',
                   'wifi-ac2', 'wifi-ac3', 'wins-server1',
                   'wins-server2']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def system_dhcp_server(data, fos):
    vdom = data['vdom']
    system_dhcp_server_data = data['system_dhcp_server']
    flattened_data = flatten_multilists_attributes(system_dhcp_server_data)
    filtered_data = filter_system_dhcp_server_data(flattened_data)
    if system_dhcp_server_data['state'] == "present":
        return fos.set('system.dhcp',
                       'server',
                       data=filtered_data,
                       vdom=vdom)

    elif system_dhcp_server_data['state'] == "absent":
        return fos.delete('system.dhcp',
                          'server',
                          mkey=filtered_data['id'],
                          vdom=vdom)


def fortios_system_dhcp(data, fos):
    login(data)

    if data['system_dhcp_server']:
        resp = system_dhcp_server(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "system_dhcp_server": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "auto-configuration": {"required": False, "type": "str",
                                       "choices": ["disable", "enable"]},
                "conflicted-ip-timeout": {"required": False, "type": "int"},
                "ddns-auth": {"required": False, "type": "str",
                              "choices": ["disable", "tsig"]},
                "ddns-key": {"required": False, "type": "str"},
                "ddns-keyname": {"required": False, "type": "str"},
                "ddns-server-ip": {"required": False, "type": "str"},
                "ddns-ttl": {"required": False, "type": "int"},
                "ddns-update": {"required": False, "type": "str",
                                "choices": ["disable", "enable"]},
                "ddns-update-override": {"required": False, "type": "str",
                                         "choices": ["disable", "enable"]},
                "ddns-zone": {"required": False, "type": "str"},
                "default-gateway": {"required": False, "type": "str"},
                "dns-server1": {"required": False, "type": "str"},
                "dns-server2": {"required": False, "type": "str"},
                "dns-server3": {"required": False, "type": "str"},
                "dns-service": {"required": False, "type": "str",
                                "choices": ["local", "default", "specify"]},
                "domain": {"required": False, "type": "str"},
                "exclude-range": {"required": False, "type": "list",
                                  "options": {
                                      "end-ip": {"required": False, "type": "str"},
                                      "id": {"required": True, "type": "int"},
                                      "start-ip": {"required": False, "type": "str"}
                                  }},
                "filename": {"required": False, "type": "str"},
                "forticlient-on-net-status": {"required": False, "type": "str",
                                              "choices": ["disable", "enable"]},
                "id": {"required": True, "type": "int"},
                "interface": {"required": False, "type": "str"},
                "ip-mode": {"required": False, "type": "str",
                            "choices": ["range", "usrgrp"]},
                "ip-range": {"required": False, "type": "list",
                             "options": {
                                 "end-ip": {"required": False, "type": "str"},
                                 "id": {"required": True, "type": "int"},
                                 "start-ip": {"required": False, "type": "str"}
                             }},
                "ipsec-lease-hold": {"required": False, "type": "int"},
                "lease-time": {"required": False, "type": "int"},
                "mac-acl-default-action": {"required": False, "type": "str",
                                           "choices": ["assign", "block"]},
                "netmask": {"required": False, "type": "str"},
                "next-server": {"required": False, "type": "str"},
                "ntp-server1": {"required": False, "type": "str"},
                "ntp-server2": {"required": False, "type": "str"},
                "ntp-server3": {"required": False, "type": "str"},
                "ntp-service": {"required": False, "type": "str",
                                "choices": ["local", "default", "specify"]},
                "options": {"required": False, "type": "list",
                            "options": {
                                "code": {"required": False, "type": "int"},
                                "id": {"required": True, "type": "int"},
                                "ip": {"required": False, "type": "str"},
                                "type": {"required": False, "type": "str",
                                         "choices": ["hex", "string", "ip"]},
                                "value": {"required": False, "type": "str"}
                            }},
                "reserved-address": {"required": False, "type": "list",
                                     "options": {
                                         "action": {"required": False, "type": "str",
                                                    "choices": ["assign", "block", "reserved"]},
                                         "description": {"required": False, "type": "str"},
                                         "id": {"required": True, "type": "int"},
                                         "ip": {"required": False, "type": "str"},
                                         "mac": {"required": False, "type": "str"}
                                     }},
                "server-type": {"required": False, "type": "str",
                                "choices": ["regular", "ipsec"]},
                "status": {"required": False, "type": "str",
                           "choices": ["disable", "enable"]},
                "tftp-server": {"required": False, "type": "list",
                                "options": {
                                    "tftp-server": {"required": True, "type": "str"}
                                }},
                "timezone": {"required": False, "type": "str",
                             "choices": ["01", "02", "03",
                                         "04", "05", "81",
                                         "06", "07", "08",
                                         "09", "10", "11",
                                         "12", "13", "74",
                                         "14", "77", "15",
                                         "87", "16", "17",
                                         "18", "19", "20",
                                         "75", "21", "22",
                                         "23", "24", "80",
                                         "79", "25", "26",
                                         "27", "28", "78",
                                         "29", "30", "31",
                                         "32", "33", "34",
                                         "35", "36", "37",
                                         "38", "83", "84",
                                         "40", "85", "41",
                                         "42", "43", "39",
                                         "44", "46", "47",
                                         "51", "48", "45",
                                         "49", "50", "52",
                                         "53", "54", "55",
                                         "56", "57", "58",
                                         "59", "60", "62",
                                         "63", "61", "64",
                                         "65", "66", "67",
                                         "68", "69", "70",
                                         "71", "72", "00",
                                         "82", "73", "86",
                                         "76"]},
                "timezone-option": {"required": False, "type": "str",
                                    "choices": ["disable", "default", "specify"]},
                "vci-match": {"required": False, "type": "str",
                              "choices": ["disable", "enable"]},
                "vci-string": {"required": False, "type": "list",
                               "options": {
                                   "vci-string": {"required": True, "type": "str"}
                               }},
                "wifi-ac1": {"required": False, "type": "str"},
                "wifi-ac2": {"required": False, "type": "str"},
                "wifi-ac3": {"required": False, "type": "str"},
                "wins-server1": {"required": False, "type": "str"},
                "wins-server2": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system_dhcp(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#
# (c) 2019 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from os import path
import json

from mock import MagicMock, call

from units.compat import unittest
from ansible.plugins.cliconf import ios
from ansible.module_utils._text import to_bytes


b_FIXTURE_DIR = b'%s/fixtures/ios' % (
    to_bytes(path.dirname(path.abspath(__file__)), errors='surrogate_or_strict')
)


def _connection_side_effect(*args, **kwargs):
    try:
        if args:
            value = args[0]
        else:
            value = kwargs.get('command')

        fixture_path = path.abspath(
            b'%s/%s' % (b_FIXTURE_DIR, b'_'.join(value.split(b' ')))
        )
        with open(fixture_path, 'rb') as file_desc:
            return file_desc.read()
    except (OSError, IOError):
        if args:
            value = args[0]
            return value
        elif kwargs.get('command'):
            value = kwargs.get('command')
            return value

        return 'Nope'


class TestPluginCLIConfIOS(unittest.TestCase):
    """ Test class for IOS CLI Conf Methods
    """
    def setUp(self):
        self._mock_connection = MagicMock()
        self._mock_connection.send.side_effect = _connection_side_effect
        self._cliconf = ios.Cliconf(self._mock_connection)
        self.maxDiff = None

    def tearDown(self):
        pass

    def test_get_device_info(self):
        """ Test get_device_info
        """
        device_info = self._cliconf.get_device_info()

        mock_device_info = {'network_os': 'ios',
                            'network_os_model': 'CSR1000V',
                            'network_os_version': '16.06.01',
                            'network_os_hostname': 'an-csr-01',
                            'network_os_image': 'bootflash:packages.conf'
                            }

        self.assertEqual(device_info, mock_device_info)

    def test_get_capabilities(self):
        """ Test get_capabilities
        """
        capabilities = json.loads(self._cliconf.get_capabilities())
        mock_capabilities = {
            'network_api': 'cliconf',
            'rpc': [
                'get_config',
                'edit_config',
                'get_capabilities',
                'get',
                'enable_response_logging',
                'disable_response_logging',
                'edit_banner',
                'get_diff',
                'run_commands',
                'get_defaults_flag'
            ],
            'device_operations': {
                'supports_diff_replace': True,
                'supports_commit': False,
                'supports_rollback': False,
                'supports_defaults': True,
                'supports_onbox_diff': False,
                'supports_commit_comment': False,
                'supports_multiline_delimiter': True,
                'supports_diff_match': True,
                'supports_diff_ignore_lines': True,
                'supports_generate_diff': True,
                'supports_replace': False
            },
            'device_info': {
                'network_os_hostname': 'an-csr-01',
                'network_os_image': 'bootflash:packages.conf',
                'network_os_model': 'CSR1000V',
                'network_os_version': '16.06.01',
                'network_os': 'ios'
            },
            'format': ['text'],
            'diff_match': ['line', 'strict', 'exact', 'none'],
            'diff_replace': ['line', 'block'],
            'output': []
        }

        self.assertEqual(
            mock_capabilities,
            capabilities
        )



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_interface \
    import NetAppOntapInterface as interface_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.type = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'interface':
            xml = self.build_interface_info(self.params)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_interface_info(data):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'net-interface-info': {
                    'interface-name': data['name'],
                    'administrative-status': data['administrative-status'],
                    'failover-policy': data['failover-policy'],
                    'firewall-policy': data['firewall-policy'],
                    'is-auto-revert': data['is-auto-revert'],
                    'home-node': data['home_node'],
                    'home-port': data['home_port'],
                    'address': data['address'],
                    'netmask': data['netmask'],
                    'role': data['role'],
                    'protocols': data['protocols'] if data.get('protocols') else None
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.mock_interface = {
            'name': 'test_lif',
            'administrative-status': 'up',
            'failover-policy': 'up',
            'firewall-policy': 'up',
            'is-auto-revert': 'true',
            'home_node': 'node',
            'role': 'test',
            'home_port': 'e0c',
            'address': '2.2.2.2',
            'netmask': '1.1.1.1',
        }

    def mock_args(self):
        return {
            'vserver': 'vserver',
            'interface_name': self.mock_interface['name'],
            'home_node': self.mock_interface['home_node'],
            'role': self.mock_interface['role'],
            'home_port': self.mock_interface['home_port'],
            'address': self.mock_interface['address'],
            'netmask': self.mock_interface['netmask'],
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password',
        }

    def get_interface_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_interface object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_interface object
        """
        interface_obj = interface_module()
        interface_obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            interface_obj.server = MockONTAPConnection()
        else:
            interface_obj.server = MockONTAPConnection(kind=kind, data=self.mock_interface)
        return interface_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            interface_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_create_error_missing_param(self):
        ''' Test if create throws an error if required param 'role' is not specified'''
        data = self.mock_args()
        del data['role']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_interface_mock_object('interface').create_interface()
        msg = 'Error: Missing one or more required parameters for creating interface: ' \
              'home_port, netmask, role, home_node, address'
        expected = sorted(','.split(msg))
        received = sorted(','.split(exc.value.args[0]['msg']))
        assert expected == received

    def test_get_nonexistent_interface(self):
        ''' Test if get_interface returns None for non-existent interface '''
        set_module_args(self.mock_args())
        result = self.get_interface_mock_object().get_interface()
        assert result is None

    def test_get_existing_interface(self):
        ''' Test if get_interface returns None for existing interface '''
        set_module_args(self.mock_args())
        result = self.get_interface_mock_object(kind='interface').get_interface()
        assert result['interface_name'] == self.mock_interface['name']

    def test_successful_create(self):
        ''' Test successful create '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object().apply()
        assert exc.value.args[0]['changed']

    def test_successful_create_for_NVMe(self):
        ''' Test successful create for NVMe protocol'''
        data = self.mock_args()
        data['protocols'] = 'fc-nvme'
        del data['address']
        del data['netmask']
        del data['home_port']
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object().apply()
        assert exc.value.args[0]['changed']

    def test_create_idempotency_for_NVMe(self):
        ''' Test create idempotency for NVMe protocol '''
        data = self.mock_args()
        data['protocols'] = 'fc-nvme'
        del data['address']
        del data['netmask']
        del data['home_port']
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object('interface').apply()
        assert not exc.value.args[0]['changed']

    def test_create_error_for_NVMe(self):
        ''' Test if create throws an error if required param 'protocols' uses NVMe'''
        data = self.mock_args()
        data['protocols'] = 'fc-nvme'
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_interface_mock_object('interface').create_interface()
        msg = 'Error: Following parameters for creating interface are not supported for data-protocol fc-nvme: ' \
              'netmask, firewall_policy, address'
        expected = sorted(','.split(msg))
        received = sorted(','.split(exc.value.args[0]['msg']))
        assert expected == received

    def test_create_idempotency(self):
        ''' Test create idempotency '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object('interface').apply()
        assert not exc.value.args[0]['changed']

    def test_successful_delete(self):
        ''' Test delete existing interface '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object('interface').apply()
        assert exc.value.args[0]['changed']

    def test_delete_idempotency(self):
        ''' Test delete idempotency '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object().apply()
        assert not exc.value.args[0]['changed']

    def test_successful_modify(self):
        ''' Test successful modify interface_minutes '''
        data = self.mock_args()
        data['home_port'] = 'new_port'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            interface_obj = self.get_interface_mock_object('interface')
            interface_obj.apply()
        assert exc.value.args[0]['changed']

    def test_modify_idempotency(self):
        ''' Test modify idempotency '''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_interface_mock_object('interface').apply()
        assert not exc.value.args[0]['changed']



# (c) 2018, NetApp Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from ansible.module_utils.six.moves.urllib.error import URLError

from ansible.module_utils.netapp import NetAppESeriesModule
from units.modules.utils import ModuleTestCase, set_module_args, AnsibleFailJson

__metaclass__ = type
from units.compat import mock


class StubNetAppESeriesModule(NetAppESeriesModule):
    def __init__(self):
        super(StubNetAppESeriesModule, self).__init__(ansible_options={})


class NetappTest(ModuleTestCase):
    REQUIRED_PARAMS = {"api_username": "rw",
                       "api_password": "password",
                       "api_url": "http://localhost",
                       "ssid": "1"}
    REQ_FUNC = "ansible.module_utils.netapp.request"

    def _set_args(self, args=None):
        module_args = self.REQUIRED_PARAMS.copy()
        if args is not None:
            module_args.update(args)
        set_module_args(module_args)

    def test_about_url_pass(self):
        """Verify about_url property returns expected about url."""
        test_set = [("http://localhost/devmgr/v2", "http://localhost:8080/devmgr/utils/about"),
                    ("http://localhost:8443/devmgr/v2", "https://localhost:8443/devmgr/utils/about"),
                    ("http://localhost:8443/devmgr/v2/", "https://localhost:8443/devmgr/utils/about"),
                    ("http://localhost:443/something_else", "https://localhost:8443/devmgr/utils/about"),
                    ("http://localhost:8443", "https://localhost:8443/devmgr/utils/about"),
                    ("http://localhost", "http://localhost:8080/devmgr/utils/about")]

        for url in test_set:
            self._set_args({"api_url": url[0]})
            base = StubNetAppESeriesModule()
            self.assertTrue(base._about_url == url[1])

    def test_is_embedded_embedded_pass(self):
        """Verify is_embedded successfully returns True when an embedded web service's rest api is inquired."""
        self._set_args()
        with mock.patch(self.REQ_FUNC, return_value=(200, {"runningAsProxy": False})):
            base = StubNetAppESeriesModule()
            self.assertTrue(base.is_embedded())
        with mock.patch(self.REQ_FUNC, return_value=(200, {"runningAsProxy": True})):
            base = StubNetAppESeriesModule()
            self.assertFalse(base.is_embedded())

    def test_check_web_services_version_pass(self):
        """Verify that an acceptable rest api version passes."""
        minimum_required = "02.10.9000.0010"
        test_set = ["03.9.9000.0010", "03.10.9000.0009", "02.11.9000.0009", "02.10.9000.0010"]

        self._set_args()
        base = StubNetAppESeriesModule()
        base.web_services_version = minimum_required
        base.is_embedded = lambda: True
        for current_version in test_set:
            with mock.patch(self.REQ_FUNC, return_value=(200, {"version": current_version})):
                self.assertTrue(base._is_web_services_valid())

    def test_check_web_services_version_fail(self):
        """Verify that an unacceptable rest api version fails."""
        minimum_required = "02.10.9000.0010"
        test_set = ["02.10.9000.0009", "02.09.9000.0010", "01.10.9000.0010"]

        self._set_args()
        base = StubNetAppESeriesModule()
        base.web_services_version = minimum_required
        base.is_embedded = lambda: True
        for current_version in test_set:
            with mock.patch(self.REQ_FUNC, return_value=(200, {"version": current_version})):
                with self.assertRaisesRegexp(AnsibleFailJson, r"version does not meet minimum version required."):
                    base._is_web_services_valid()

    def test_is_embedded_fail(self):
        """Verify exception is thrown when a web service's rest api fails to return about information."""
        self._set_args()
        with mock.patch(self.REQ_FUNC, return_value=Exception()):
            with self.assertRaisesRegexp(AnsibleFailJson, r"Failed to retrieve the webservices about information!"):
                base = StubNetAppESeriesModule()
                base.is_embedded()
        with mock.patch(self.REQ_FUNC, side_effect=[URLError(""), Exception()]):
            with self.assertRaisesRegexp(AnsibleFailJson, r"Failed to retrieve the webservices about information!"):
                base = StubNetAppESeriesModule()
                base.is_embedded()

    def test_tweak_url_pass(self):
        """Verify a range of valid netapp eseries rest api urls pass."""
        test_set = [("http://localhost/devmgr/v2", "http://localhost:8080/devmgr/v2/"),
                    ("localhost", "https://localhost:8443/devmgr/v2/"),
                    ("localhost:8443/devmgr/v2", "https://localhost:8443/devmgr/v2/"),
                    ("https://localhost/devmgr/v2", "https://localhost:8443/devmgr/v2/"),
                    ("http://localhost:8443", "https://localhost:8443/devmgr/v2/"),
                    ("http://localhost:/devmgr/v2", "https://localhost:8443/devmgr/v2/"),
                    ("http://localhost:8080", "http://localhost:8080/devmgr/v2/"),
                    ("http://localhost", "http://localhost:8080/devmgr/v2/"),
                    ("localhost/devmgr/v2", "https://localhost:8443/devmgr/v2/"),
                    ("localhost/devmgr", "https://localhost:8443/devmgr/v2/"),
                    ("localhost/devmgr/v3", "https://localhost:8443/devmgr/v2/"),
                    ("localhost/something", "https://localhost:8443/devmgr/v2/"),
                    ("ftp://localhost", "https://localhost:8443/devmgr/v2/"),
                    ("ftp://localhost:8080", "http://localhost:8080/devmgr/v2/"),
                    ("ftp://localhost/devmgr/v2/", "https://localhost:8443/devmgr/v2/")]

        for test in test_set:
            self._set_args({"api_url": test[0]})
            with mock.patch(self.REQ_FUNC, side_effect=[URLError(""), (200, {"runningAsProxy": False})]):
                base = StubNetAppESeriesModule()
                base._tweak_url()
                self.assertTrue(base.url == test[1])

    def test_check_url_missing_hostname_fail(self):
        """Verify exception is thrown when hostname or ip address is missing."""
        self._set_args({"api_url": "http:///devmgr/v2"})
        with mock.patch(self.REQ_FUNC, return_value=(200, {"runningAsProxy": True})):
            with self.assertRaisesRegexp(AnsibleFailJson, r"Failed to provide a valid hostname or IP address."):
                base = StubNetAppESeriesModule()
                base._tweak_url()



# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json

from units.compat.mock import patch
from ansible.modules.network.netvisor import pn_admin_service
from units.modules.utils import set_module_args
from .nvos_module import TestNvosModule, load_fixture


class TestAdminServiceModule(TestNvosModule):

    module = pn_admin_service

    def setUp(self):
        self.mock_run_nvos_commands = patch('ansible.modules.network.netvisor.pn_admin_service.run_cli')
        self.run_nvos_commands = self.mock_run_nvos_commands.start()

    def tearDown(self):
        self.mock_run_nvos_commands.stop()

    def run_cli_patch(self, module, cli, state_map):
        if state_map['update'] == 'admin-service-modify':
            results = dict(
                changed=True,
                cli_cmd=cli
            )
        module.exit_json(**results)

    def load_fixtures(self, commands=None, state=None, transport='cli'):
        self.run_nvos_commands.side_effect = self.run_cli_patch

    def test_admin_service_modify_t1(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn__if': 'mgmt',
                         'pn_web': 'False', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 admin-service-modify  if mgmt no-web '
        self.assertEqual(result['cli_cmd'], expected_cmd)

    def test_admin_service_modify_t2(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn__if': 'mgmt',
                         'pn_snmp': 'True', 'pn_net_api': 'True', 'pn_ssh': 'True', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 admin-service-modify  if mgmt snmp  ssh  net-api '
        self.assertEqual(result['cli_cmd'], expected_cmd)

    def test_admin_service_modify_t3(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn__if': 'data',
                         'pn_web_port': '8080', 'pn_net_api': 'True', 'pn_web_log': 'True', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 admin-service-modify  if data web-port 8080 net-api  web-log '
        self.assertEqual(result['cli_cmd'], expected_cmd)



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_policy6
short_description: Configure IPv6 policies in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and policy6 category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_policy6:
        description:
            - Configure IPv6 policies.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            action:
                description:
                    - Policy action (allow/deny/ipsec).
                choices:
                    - accept
                    - deny
                    - ipsec
            app-category:
                description:
                    - Application category ID list.
                suboptions:
                    id:
                        description:
                            - Category IDs.
                        required: true
            app-group:
                description:
                    - Application group names.
                suboptions:
                    name:
                        description:
                            - Application group names. Source application.group.name.
                        required: true
            application:
                description:
                    - Application ID list.
                suboptions:
                    id:
                        description:
                            - Application IDs.
                        required: true
            application-list:
                description:
                    - Name of an existing Application list. Source application.list.name.
            av-profile:
                description:
                    - Name of an existing Antivirus profile. Source antivirus.profile.name.
            comments:
                description:
                    - Comment.
            custom-log-fields:
                description:
                    - Log field index numbers to append custom log fields to log messages for this policy.
                suboptions:
                    field-id:
                        description:
                            - Custom log field. Source log.custom-field.id.
                        required: true
            devices:
                description:
                    - Names of devices or device groups that can be matched by the policy.
                suboptions:
                    name:
                        description:
                            - Device or group name. Source user.device.alias user.device-group.name user.device-category.name.
                        required: true
            diffserv-forward:
                description:
                    - Enable to change packet's DiffServ values to the specified diffservcode-forward value.
                choices:
                    - enable
                    - disable
            diffserv-reverse:
                description:
                    - Enable to change packet's reverse (reply) DiffServ values to the specified diffservcode-rev value.
                choices:
                    - enable
                    - disable
            diffservcode-forward:
                description:
                    - Change packet's DiffServ to this value.
            diffservcode-rev:
                description:
                    - Change packet's reverse (reply) DiffServ to this value.
            dlp-sensor:
                description:
                    - Name of an existing DLP sensor. Source dlp.sensor.name.
            dscp-match:
                description:
                    - Enable DSCP check.
                choices:
                    - enable
                    - disable
            dscp-negate:
                description:
                    - Enable negated DSCP match.
                choices:
                    - enable
                    - disable
            dscp-value:
                description:
                    - DSCP value.
            dsri:
                description:
                    - Enable DSRI to ignore HTTP server responses.
                choices:
                    - enable
                    - disable
            dstaddr:
                description:
                    - Destination address and address group names.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name firewall.vip6.name firewall.vipgrp6.name.
                        required: true
            dstaddr-negate:
                description:
                    - When enabled dstaddr specifies what the destination address must NOT be.
                choices:
                    - enable
                    - disable
            dstintf:
                description:
                    - Outgoing (egress) interface.
                suboptions:
                    name:
                        description:
                            - Interface name. Source system.interface.name system.zone.name.
                        required: true
            firewall-session-dirty:
                description:
                    - How to handle sessions if the configuration of this firewall policy changes.
                choices:
                    - check-all
                    - check-new
            fixedport:
                description:
                    - Enable to prevent source NAT from changing a session's source port.
                choices:
                    - enable
                    - disable
            global-label:
                description:
                    - Label for the policy that appears when the GUI is in Global View mode.
            groups:
                description:
                    - Names of user groups that can authenticate with this policy.
                suboptions:
                    name:
                        description:
                            - Group name. Source user.group.name.
                        required: true
            icap-profile:
                description:
                    - Name of an existing ICAP profile. Source icap.profile.name.
            inbound:
                description:
                    - "Policy-based IPsec VPN: only traffic from the remote network can initiate a VPN."
                choices:
                    - enable
                    - disable
            ippool:
                description:
                    - Enable to use IP Pools for source NAT.
                choices:
                    - enable
                    - disable
            ips-sensor:
                description:
                    - Name of an existing IPS sensor. Source ips.sensor.name.
            label:
                description:
                    - Label for the policy that appears when the GUI is in Section View mode.
            logtraffic:
                description:
                    - Enable or disable logging. Log all sessions or security profile sessions.
                choices:
                    - all
                    - utm
                    - disable
            logtraffic-start:
                description:
                    - Record logs when a session starts and ends.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - Policy name.
            nat:
                description:
                    - Enable/disable source NAT.
                choices:
                    - enable
                    - disable
            natinbound:
                description:
                    - "Policy-based IPsec VPN: apply destination NAT to inbound traffic."
                choices:
                    - enable
                    - disable
            natoutbound:
                description:
                    - "Policy-based IPsec VPN: apply source NAT to outbound traffic."
                choices:
                    - enable
                    - disable
            outbound:
                description:
                    - "Policy-based IPsec VPN: only traffic from the internal network can initiate a VPN."
                choices:
                    - enable
                    - disable
            per-ip-shaper:
                description:
                    - Per-IP traffic shaper. Source firewall.shaper.per-ip-shaper.name.
            policyid:
                description:
                    - Policy ID.
                required: true
            poolname:
                description:
                    - IP Pool names.
                suboptions:
                    name:
                        description:
                            - IP pool name. Source firewall.ippool6.name.
                        required: true
            profile-group:
                description:
                    - Name of profile group. Source firewall.profile-group.name.
            profile-protocol-options:
                description:
                    - Name of an existing Protocol options profile. Source firewall.profile-protocol-options.name.
            profile-type:
                description:
                    - Determine whether the firewall policy allows security profile groups or single profiles only.
                choices:
                    - single
                    - group
            replacemsg-override-group:
                description:
                    - Override the default replacement message group for this policy. Source system.replacemsg-group.name.
            rsso:
                description:
                    - Enable/disable RADIUS single sign-on (RSSO).
                choices:
                    - enable
                    - disable
            schedule:
                description:
                    - Schedule name. Source firewall.schedule.onetime.name firewall.schedule.recurring.name firewall.schedule.group.name.
            send-deny-packet:
                description:
                    - Enable/disable return of deny-packet.
                choices:
                    - enable
                    - disable
            service:
                description:
                    - Service and service group names.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.service.custom.name firewall.service.group.name.
                        required: true
            service-negate:
                description:
                    - When enabled service specifies what the service must NOT be.
                choices:
                    - enable
                    - disable
            session-ttl:
                description:
                    - Session TTL in seconds for sessions accepted by this policy. 0 means use the system default session TTL.
            spamfilter-profile:
                description:
                    - Name of an existing Spam filter profile. Source spamfilter.profile.name.
            srcaddr:
                description:
                    - Source address and address group names.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            srcaddr-negate:
                description:
                    - When enabled srcaddr specifies what the source address must NOT be.
                choices:
                    - enable
                    - disable
            srcintf:
                description:
                    - Incoming (ingress) interface.
                suboptions:
                    name:
                        description:
                            - Interface name. Source system.zone.name system.interface.name.
                        required: true
            ssh-filter-profile:
                description:
                    - Name of an existing SSH filter profile. Source ssh-filter.profile.name.
            ssl-mirror:
                description:
                    - Enable to copy decrypted SSL traffic to a FortiGate interface (called SSL mirroring).
                choices:
                    - enable
                    - disable
            ssl-mirror-intf:
                description:
                    - SSL mirror interface name.
                suboptions:
                    name:
                        description:
                            - Interface name. Source system.zone.name system.interface.name.
                        required: true
            ssl-ssh-profile:
                description:
                    - Name of an existing SSL SSH profile. Source firewall.ssl-ssh-profile.name.
            status:
                description:
                    - Enable or disable this policy.
                choices:
                    - enable
                    - disable
            tcp-mss-receiver:
                description:
                    - Receiver TCP maximum segment size (MSS).
            tcp-mss-sender:
                description:
                    - Sender TCP maximum segment size (MSS).
            tcp-session-without-syn:
                description:
                    - Enable/disable creation of TCP session without SYN flag.
                choices:
                    - all
                    - data-only
                    - disable
            timeout-send-rst:
                description:
                    - Enable/disable sending RST packets when TCP sessions expire.
                choices:
                    - enable
                    - disable
            traffic-shaper:
                description:
                    - Reverse traffic shaper. Source firewall.shaper.traffic-shaper.name.
            traffic-shaper-reverse:
                description:
                    - Reverse traffic shaper. Source firewall.shaper.traffic-shaper.name.
            url-category:
                description:
                    - URL category ID list.
                suboptions:
                    id:
                        description:
                            - URL category ID.
                        required: true
            users:
                description:
                    - Names of individual users that can authenticate with this policy.
                suboptions:
                    name:
                        description:
                            - Names of individual users that can authenticate with this policy. Source user.local.name.
                        required: true
            utm-status:
                description:
                    - Enable AV/web/ips protection profile.
                choices:
                    - enable
                    - disable
            uuid:
                description:
                    - Universally Unique Identifier (UUID; automatically assigned but can be manually reset).
            vlan-cos-fwd:
                description:
                    - "VLAN forward direction user priority: 255 passthrough, 0 lowest, 7 highest"
            vlan-cos-rev:
                description:
                    - "VLAN reverse direction user priority: 255 passthrough, 0 lowest, 7 highest"
            vlan-filter:
                description:
                    - Set VLAN filters.
            voip-profile:
                description:
                    - Name of an existing VoIP profile. Source voip.profile.name.
            vpntunnel:
                description:
                    - "Policy-based IPsec VPN: name of the IPsec VPN Phase 1. Source vpn.ipsec.phase1.name vpn.ipsec.manualkey.name."
            webfilter-profile:
                description:
                    - Name of an existing Web filter profile. Source webfilter.profile.name.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv6 policies.
    fortios_firewall_policy6:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_policy6:
        state: "present"
        action: "accept"
        app-category:
         -
            id:  "5"
        app-group:
         -
            name: "default_name_7 (source application.group.name)"
        application:
         -
            id:  "9"
        application-list: "<your_own_value> (source application.list.name)"
        av-profile: "<your_own_value> (source antivirus.profile.name)"
        comments: "<your_own_value>"
        custom-log-fields:
         -
            field-id: "<your_own_value> (source log.custom-field.id)"
        devices:
         -
            name: "default_name_16 (source user.device.alias user.device-group.name user.device-category.name)"
        diffserv-forward: "enable"
        diffserv-reverse: "enable"
        diffservcode-forward: "<your_own_value>"
        diffservcode-rev: "<your_own_value>"
        dlp-sensor: "<your_own_value> (source dlp.sensor.name)"
        dscp-match: "enable"
        dscp-negate: "enable"
        dscp-value: "<your_own_value>"
        dsri: "enable"
        dstaddr:
         -
            name: "default_name_27 (source firewall.address6.name firewall.addrgrp6.name firewall.vip6.name firewall.vipgrp6.name)"
        dstaddr-negate: "enable"
        dstintf:
         -
            name: "default_name_30 (source system.interface.name system.zone.name)"
        firewall-session-dirty: "check-all"
        fixedport: "enable"
        global-label: "<your_own_value>"
        groups:
         -
            name: "default_name_35 (source user.group.name)"
        icap-profile: "<your_own_value> (source icap.profile.name)"
        inbound: "enable"
        ippool: "enable"
        ips-sensor: "<your_own_value> (source ips.sensor.name)"
        label: "<your_own_value>"
        logtraffic: "all"
        logtraffic-start: "enable"
        name: "default_name_43"
        nat: "enable"
        natinbound: "enable"
        natoutbound: "enable"
        outbound: "enable"
        per-ip-shaper: "<your_own_value> (source firewall.shaper.per-ip-shaper.name)"
        policyid: "49"
        poolname:
         -
            name: "default_name_51 (source firewall.ippool6.name)"
        profile-group: "<your_own_value> (source firewall.profile-group.name)"
        profile-protocol-options: "<your_own_value> (source firewall.profile-protocol-options.name)"
        profile-type: "single"
        replacemsg-override-group: "<your_own_value> (source system.replacemsg-group.name)"
        rsso: "enable"
        schedule: "<your_own_value> (source firewall.schedule.onetime.name firewall.schedule.recurring.name firewall.schedule.group.name)"
        send-deny-packet: "enable"
        service:
         -
            name: "default_name_60 (source firewall.service.custom.name firewall.service.group.name)"
        service-negate: "enable"
        session-ttl: "62"
        spamfilter-profile: "<your_own_value> (source spamfilter.profile.name)"
        srcaddr:
         -
            name: "default_name_65 (source firewall.address6.name firewall.addrgrp6.name)"
        srcaddr-negate: "enable"
        srcintf:
         -
            name: "default_name_68 (source system.zone.name system.interface.name)"
        ssh-filter-profile: "<your_own_value> (source ssh-filter.profile.name)"
        ssl-mirror: "enable"
        ssl-mirror-intf:
         -
            name: "default_name_72 (source system.zone.name system.interface.name)"
        ssl-ssh-profile: "<your_own_value> (source firewall.ssl-ssh-profile.name)"
        status: "enable"
        tcp-mss-receiver: "75"
        tcp-mss-sender: "76"
        tcp-session-without-syn: "all"
        timeout-send-rst: "enable"
        traffic-shaper: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
        traffic-shaper-reverse: "<your_own_value> (source firewall.shaper.traffic-shaper.name)"
        url-category:
         -
            id:  "82"
        users:
         -
            name: "default_name_84 (source user.local.name)"
        utm-status: "enable"
        uuid: "<your_own_value>"
        vlan-cos-fwd: "87"
        vlan-cos-rev: "88"
        vlan-filter: "<your_own_value>"
        voip-profile: "<your_own_value> (source voip.profile.name)"
        vpntunnel: "<your_own_value> (source vpn.ipsec.phase1.name vpn.ipsec.manualkey.name)"
        webfilter-profile: "<your_own_value> (source webfilter.profile.name)"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_policy6_data(json):
    option_list = ['action', 'app-category', 'app-group',
                   'application', 'application-list', 'av-profile',
                   'comments', 'custom-log-fields', 'devices',
                   'diffserv-forward', 'diffserv-reverse', 'diffservcode-forward',
                   'diffservcode-rev', 'dlp-sensor', 'dscp-match',
                   'dscp-negate', 'dscp-value', 'dsri',
                   'dstaddr', 'dstaddr-negate', 'dstintf',
                   'firewall-session-dirty', 'fixedport', 'global-label',
                   'groups', 'icap-profile', 'inbound',
                   'ippool', 'ips-sensor', 'label',
                   'logtraffic', 'logtraffic-start', 'name',
                   'nat', 'natinbound', 'natoutbound',
                   'outbound', 'per-ip-shaper', 'policyid',
                   'poolname', 'profile-group', 'profile-protocol-options',
                   'profile-type', 'replacemsg-override-group', 'rsso',
                   'schedule', 'send-deny-packet', 'service',
                   'service-negate', 'session-ttl', 'spamfilter-profile',
                   'srcaddr', 'srcaddr-negate', 'srcintf',
                   'ssh-filter-profile', 'ssl-mirror', 'ssl-mirror-intf',
                   'ssl-ssh-profile', 'status', 'tcp-mss-receiver',
                   'tcp-mss-sender', 'tcp-session-without-syn', 'timeout-send-rst',
                   'traffic-shaper', 'traffic-shaper-reverse', 'url-category',
                   'users', 'utm-status', 'uuid',
                   'vlan-cos-fwd', 'vlan-cos-rev', 'vlan-filter',
                   'voip-profile', 'vpntunnel', 'webfilter-profile']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_policy6(data, fos):
    vdom = data['vdom']
    firewall_policy6_data = data['firewall_policy6']
    filtered_data = filter_firewall_policy6_data(firewall_policy6_data)
    if firewall_policy6_data['state'] == "present":
        return fos.set('firewall',
                       'policy6',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_policy6_data['state'] == "absent":
        return fos.delete('firewall',
                          'policy6',
                          mkey=filtered_data['policyid'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_policy6']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_policy6": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "action": {"required": False, "type": "str",
                           "choices": ["accept", "deny", "ipsec"]},
                "app-category": {"required": False, "type": "list",
                                 "options": {
                                     "id": {"required": True, "type": "int"}
                                 }},
                "app-group": {"required": False, "type": "list",
                              "options": {
                                  "name": {"required": True, "type": "str"}
                              }},
                "application": {"required": False, "type": "list",
                                "options": {
                                    "id": {"required": True, "type": "int"}
                                }},
                "application-list": {"required": False, "type": "str"},
                "av-profile": {"required": False, "type": "str"},
                "comments": {"required": False, "type": "str"},
                "custom-log-fields": {"required": False, "type": "list",
                                      "options": {
                                          "field-id": {"required": True, "type": "str"}
                                      }},
                "devices": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "diffserv-forward": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "diffserv-reverse": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "diffservcode-forward": {"required": False, "type": "str"},
                "diffservcode-rev": {"required": False, "type": "str"},
                "dlp-sensor": {"required": False, "type": "str"},
                "dscp-match": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "dscp-negate": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "dscp-value": {"required": False, "type": "str"},
                "dsri": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "dstaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "dstaddr-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "dstintf": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "firewall-session-dirty": {"required": False, "type": "str",
                                           "choices": ["check-all", "check-new"]},
                "fixedport": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "global-label": {"required": False, "type": "str"},
                "groups": {"required": False, "type": "list",
                           "options": {
                               "name": {"required": True, "type": "str"}
                           }},
                "icap-profile": {"required": False, "type": "str"},
                "inbound": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "ippool": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "ips-sensor": {"required": False, "type": "str"},
                "label": {"required": False, "type": "str"},
                "logtraffic": {"required": False, "type": "str",
                               "choices": ["all", "utm", "disable"]},
                "logtraffic-start": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "name": {"required": False, "type": "str"},
                "nat": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "natinbound": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "natoutbound": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "outbound": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "per-ip-shaper": {"required": False, "type": "str"},
                "policyid": {"required": True, "type": "int"},
                "poolname": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "profile-group": {"required": False, "type": "str"},
                "profile-protocol-options": {"required": False, "type": "str"},
                "profile-type": {"required": False, "type": "str",
                                 "choices": ["single", "group"]},
                "replacemsg-override-group": {"required": False, "type": "str"},
                "rsso": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "schedule": {"required": False, "type": "str"},
                "send-deny-packet": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "service": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "service-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "session-ttl": {"required": False, "type": "int"},
                "spamfilter-profile": {"required": False, "type": "str"},
                "srcaddr": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "srcaddr-negate": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "srcintf": {"required": False, "type": "list",
                            "options": {
                                "name": {"required": True, "type": "str"}
                            }},
                "ssh-filter-profile": {"required": False, "type": "str"},
                "ssl-mirror": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "ssl-mirror-intf": {"required": False, "type": "list",
                                    "options": {
                                        "name": {"required": True, "type": "str"}
                                    }},
                "ssl-ssh-profile": {"required": False, "type": "str"},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "tcp-mss-receiver": {"required": False, "type": "int"},
                "tcp-mss-sender": {"required": False, "type": "int"},
                "tcp-session-without-syn": {"required": False, "type": "str",
                                            "choices": ["all", "data-only", "disable"]},
                "timeout-send-rst": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "traffic-shaper": {"required": False, "type": "str"},
                "traffic-shaper-reverse": {"required": False, "type": "str"},
                "url-category": {"required": False, "type": "list",
                                 "options": {
                                     "id": {"required": True, "type": "int"}
                                 }},
                "users": {"required": False, "type": "list",
                          "options": {
                              "name": {"required": True, "type": "str"}
                          }},
                "utm-status": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "uuid": {"required": False, "type": "str"},
                "vlan-cos-fwd": {"required": False, "type": "int"},
                "vlan-cos-rev": {"required": False, "type": "int"},
                "vlan-filter": {"required": False, "type": "str"},
                "voip-profile": {"required": False, "type": "str"},
                "vpntunnel": {"required": False, "type": "str"},
                "webfilter-profile": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_vpn_ssl_web_portal
short_description: Portal in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify vpn_ssl_web feature and portal category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    vpn_ssl_web_portal:
        description:
            - Portal.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            allow-user-access:
                description:
                    - Allow user access to SSL-VPN applications.
                choices:
                    - web
                    - ftp
                    - smb
                    - telnet
                    - ssh
                    - vnc
                    - rdp
                    - ping
                    - citrix
                    - portforward
            auto-connect:
                description:
                    - Enable/disable automatic connect by client when system is up.
                choices:
                    - enable
                    - disable
            bookmark-group:
                description:
                    - Portal bookmark group.
                suboptions:
                    bookmarks:
                        description:
                            - Bookmark table.
                        suboptions:
                            additional-params:
                                description:
                                    - Additional parameters.
                            apptype:
                                description:
                                    - Application type.
                                choices:
                                    - citrix
                                    - ftp
                                    - portforward
                                    - rdp
                                    - smb
                                    - ssh
                                    - telnet
                                    - vnc
                                    - web
                            description:
                                description:
                                    - Description.
                            folder:
                                description:
                                    - Network shared file folder parameter.
                            form-data:
                                description:
                                    - Form data.
                                suboptions:
                                    name:
                                        description:
                                            - Name.
                                        required: true
                                    value:
                                        description:
                                            - Value.
                            host:
                                description:
                                    - Host name/IP parameter.
                            listening-port:
                                description:
                                    - Listening port (0 - 65535).
                            load-balancing-info:
                                description:
                                    - The load balancing information or cookie which should be provided to the connection broker.
                            logon-password:
                                description:
                                    - Logon password.
                            logon-user:
                                description:
                                    - Logon user.
                            name:
                                description:
                                    - Bookmark name.
                                required: true
                            port:
                                description:
                                    - Remote port.
                            preconnection-blob:
                                description:
                                    - An arbitrary string which identifies the RDP source.
                            preconnection-id:
                                description:
                                    - The numeric ID of the RDP source (0-2147483648).
                            remote-port:
                                description:
                                    - Remote port (0 - 65535).
                            security:
                                description:
                                    - Security mode for RDP connection.
                                choices:
                                    - rdp
                                    - nla
                                    - tls
                                    - any
                            server-layout:
                                description:
                                    - Server side keyboard layout.
                                choices:
                                    - de-de-qwertz
                                    - en-gb-qwerty
                                    - en-us-qwerty
                                    - es-es-qwerty
                                    - fr-fr-azerty
                                    - fr-ch-qwertz
                                    - it-it-qwerty
                                    - ja-jp-qwerty
                                    - pt-br-qwerty
                                    - sv-se-qwerty
                                    - tr-tr-qwerty
                                    - failsafe
                            show-status-window:
                                description:
                                    - Enable/disable showing of status window.
                                choices:
                                    - enable
                                    - disable
                            sso:
                                description:
                                    - Single Sign-On.
                                choices:
                                    - disable
                                    - static
                                    - auto
                            sso-credential:
                                description:
                                    - Single sign-on credentials.
                                choices:
                                    - sslvpn-login
                                    - alternative
                            sso-credential-sent-once:
                                description:
                                    - Single sign-on credentials are only sent once to remote server.
                                choices:
                                    - enable
                                    - disable
                            sso-password:
                                description:
                                    - SSO password.
                            sso-username:
                                description:
                                    - SSO user name.
                            url:
                                description:
                                    - URL parameter.
                    name:
                        description:
                            - Bookmark group name.
                        required: true
            custom-lang:
                description:
                    - Change the web portal display language. Overrides config system global set language. You can use config system custom-language and
                       execute system custom-language to add custom language files. Source system.custom-language.name.
            customize-forticlient-download-url:
                description:
                    - Enable support of customized download URL for FortiClient.
                choices:
                    - enable
                    - disable
            display-bookmark:
                description:
                    - Enable to display the web portal bookmark widget.
                choices:
                    - enable
                    - disable
            display-connection-tools:
                description:
                    - Enable to display the web portal connection tools widget.
                choices:
                    - enable
                    - disable
            display-history:
                description:
                    - Enable to display the web portal user login history widget.
                choices:
                    - enable
                    - disable
            display-status:
                description:
                    - Enable to display the web portal status widget.
                choices:
                    - enable
                    - disable
            dns-server1:
                description:
                    - IPv4 DNS server 1.
            dns-server2:
                description:
                    - IPv4 DNS server 2.
            dns-suffix:
                description:
                    - DNS suffix.
            exclusive-routing:
                description:
                    - Enable/disable all traffic go through tunnel only.
                choices:
                    - enable
                    - disable
            forticlient-download:
                description:
                    - Enable/disable download option for FortiClient.
                choices:
                    - enable
                    - disable
            forticlient-download-method:
                description:
                    - FortiClient download method.
                choices:
                    - direct
                    - ssl-vpn
            heading:
                description:
                    - Web portal heading message.
            hide-sso-credential:
                description:
                    - Enable to prevent SSO credential being sent to client.
                choices:
                    - enable
                    - disable
            host-check:
                description:
                    - Type of host checking performed on endpoints.
                choices:
                    - none
                    - av
                    - fw
                    - av-fw
                    - custom
            host-check-interval:
                description:
                    - Periodic host check interval. Value of 0 means disabled and host checking only happens when the endpoint connects.
            host-check-policy:
                description:
                    - One or more policies to require the endpoint to have specific security software.
                suboptions:
                    name:
                        description:
                            - Host check software list name. Source vpn.ssl.web.host-check-software.name.
                        required: true
            ip-mode:
                description:
                    - Method by which users of this SSL-VPN tunnel obtain IP addresses.
                choices:
                    - range
                    - user-group
            ip-pools:
                description:
                    - IPv4 firewall source address objects reserved for SSL-VPN tunnel mode clients.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            ipv6-dns-server1:
                description:
                    - IPv6 DNS server 1.
            ipv6-dns-server2:
                description:
                    - IPv6 DNS server 2.
            ipv6-exclusive-routing:
                description:
                    - Enable/disable all IPv6 traffic go through tunnel only.
                choices:
                    - enable
                    - disable
            ipv6-pools:
                description:
                    - IPv4 firewall source address objects reserved for SSL-VPN tunnel mode clients.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            ipv6-service-restriction:
                description:
                    - Enable/disable IPv6 tunnel service restriction.
                choices:
                    - enable
                    - disable
            ipv6-split-tunneling:
                description:
                    - Enable/disable IPv6 split tunneling.
                choices:
                    - enable
                    - disable
            ipv6-split-tunneling-routing-address:
                description:
                    - IPv6 SSL-VPN tunnel mode firewall address objects that override firewall policy destination addresses to control split-tunneling access.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address6.name firewall.addrgrp6.name.
                        required: true
            ipv6-tunnel-mode:
                description:
                    - Enable/disable IPv6 SSL-VPN tunnel mode.
                choices:
                    - enable
                    - disable
            ipv6-wins-server1:
                description:
                    - IPv6 WINS server 1.
            ipv6-wins-server2:
                description:
                    - IPv6 WINS server 2.
            keep-alive:
                description:
                    - Enable/disable automatic reconnect for FortiClient connections.
                choices:
                    - enable
                    - disable
            limit-user-logins:
                description:
                    - Enable to limit each user to one SSL-VPN session at a time.
                choices:
                    - enable
                    - disable
            mac-addr-action:
                description:
                    - Client MAC address action.
                choices:
                    - allow
                    - deny
            mac-addr-check:
                description:
                    - Enable/disable MAC address host checking.
                choices:
                    - enable
                    - disable
            mac-addr-check-rule:
                description:
                    - Client MAC address check rule.
                suboptions:
                    mac-addr-list:
                        description:
                            - Client MAC address list.
                        suboptions:
                            addr:
                                description:
                                    - Client MAC address.
                                required: true
                    mac-addr-mask:
                        description:
                            - Client MAC address mask.
                    name:
                        description:
                            - Client MAC address check rule name.
                        required: true
            macos-forticlient-download-url:
                description:
                    - Download URL for Mac FortiClient.
            name:
                description:
                    - Portal name.
                required: true
            os-check:
                description:
                    - Enable to let the FortiGate decide action based on client OS.
                choices:
                    - enable
                    - disable
            os-check-list:
                description:
                    - SSL VPN OS checks.
                suboptions:
                    action:
                        description:
                            - OS check options.
                        choices:
                            - deny
                            - allow
                            - check-up-to-date
                    latest-patch-level:
                        description:
                            - Latest OS patch level.
                    name:
                        description:
                            - Name.
                        required: true
                    tolerance:
                        description:
                            - OS patch level tolerance.
            redir-url:
                description:
                    - Client login redirect URL.
            save-password:
                description:
                    - Enable/disable FortiClient saving the user's password.
                choices:
                    - enable
                    - disable
            service-restriction:
                description:
                    - Enable/disable tunnel service restriction.
                choices:
                    - enable
                    - disable
            skip-check-for-unsupported-browser:
                description:
                    - Enable to skip host check if browser does not support it.
                choices:
                    - enable
                    - disable
            skip-check-for-unsupported-os:
                description:
                    - Enable to skip host check if client OS does not support it.
                choices:
                    - enable
                    - disable
            smb-ntlmv1-auth:
                description:
                    - Enable support of NTLMv1 for Samba authentication.
                choices:
                    - enable
                    - disable
            smbv1:
                description:
                    - Enable/disable support of SMBv1 for Samba.
                choices:
                    - enable
                    - disable
            split-dns:
                description:
                    - Split DNS for SSL VPN.
                suboptions:
                    dns-server1:
                        description:
                            - DNS server 1.
                    dns-server2:
                        description:
                            - DNS server 2.
                    domains:
                        description:
                            - Split DNS domains used for SSL-VPN clients separated by comma(,).
                    id:
                        description:
                            - ID.
                        required: true
                    ipv6-dns-server1:
                        description:
                            - IPv6 DNS server 1.
                    ipv6-dns-server2:
                        description:
                            - IPv6 DNS server 2.
            split-tunneling:
                description:
                    - Enable/disable IPv4 split tunneling.
                choices:
                    - enable
                    - disable
            split-tunneling-routing-address:
                description:
                    - IPv4 SSL-VPN tunnel mode firewall address objects that override firewall policy destination addresses to control split-tunneling access.
                suboptions:
                    name:
                        description:
                            - Address name. Source firewall.address.name firewall.addrgrp.name.
                        required: true
            theme:
                description:
                    - Web portal color scheme.
                choices:
                    - blue
                    - green
                    - red
                    - melongene
                    - mariner
            tunnel-mode:
                description:
                    - Enable/disable IPv4 SSL-VPN tunnel mode.
                choices:
                    - enable
                    - disable
            user-bookmark:
                description:
                    - Enable to allow web portal users to create their own bookmarks.
                choices:
                    - enable
                    - disable
            user-group-bookmark:
                description:
                    - Enable to allow web portal users to create bookmarks for all users in the same user group.
                choices:
                    - enable
                    - disable
            web-mode:
                description:
                    - Enable/disable SSL VPN web mode.
                choices:
                    - enable
                    - disable
            windows-forticlient-download-url:
                description:
                    - Download URL for Windows FortiClient.
            wins-server1:
                description:
                    - IPv4 WINS server 1.
            wins-server2:
                description:
                    - IPv4 WINS server 1.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Portal.
    fortios_vpn_ssl_web_portal:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      vpn_ssl_web_portal:
        state: "present"
        allow-user-access: "web"
        auto-connect: "enable"
        bookmark-group:
         -
            bookmarks:
             -
                additional-params: "<your_own_value>"
                apptype: "citrix"
                description: "<your_own_value>"
                folder: "<your_own_value>"
                form-data:
                 -
                    name: "default_name_12"
                    value: "<your_own_value>"
                host: "<your_own_value>"
                listening-port: "15"
                load-balancing-info: "<your_own_value>"
                logon-password: "<your_own_value>"
                logon-user: "<your_own_value>"
                name: "default_name_19"
                port: "20"
                preconnection-blob: "<your_own_value>"
                preconnection-id: "22"
                remote-port: "23"
                security: "rdp"
                server-layout: "de-de-qwertz"
                show-status-window: "enable"
                sso: "disable"
                sso-credential: "sslvpn-login"
                sso-credential-sent-once: "enable"
                sso-password: "<your_own_value>"
                sso-username: "<your_own_value>"
                url: "myurl.com"
            name: "default_name_33"
        custom-lang: "<your_own_value> (source system.custom-language.name)"
        customize-forticlient-download-url: "enable"
        display-bookmark: "enable"
        display-connection-tools: "enable"
        display-history: "enable"
        display-status: "enable"
        dns-server1: "<your_own_value>"
        dns-server2: "<your_own_value>"
        dns-suffix: "<your_own_value>"
        exclusive-routing: "enable"
        forticlient-download: "enable"
        forticlient-download-method: "direct"
        heading: "<your_own_value>"
        hide-sso-credential: "enable"
        host-check: "none"
        host-check-interval: "49"
        host-check-policy:
         -
            name: "default_name_51 (source vpn.ssl.web.host-check-software.name)"
        ip-mode: "range"
        ip-pools:
         -
            name: "default_name_54 (source firewall.address.name firewall.addrgrp.name)"
        ipv6-dns-server1: "<your_own_value>"
        ipv6-dns-server2: "<your_own_value>"
        ipv6-exclusive-routing: "enable"
        ipv6-pools:
         -
            name: "default_name_59 (source firewall.address6.name firewall.addrgrp6.name)"
        ipv6-service-restriction: "enable"
        ipv6-split-tunneling: "enable"
        ipv6-split-tunneling-routing-address:
         -
            name: "default_name_63 (source firewall.address6.name firewall.addrgrp6.name)"
        ipv6-tunnel-mode: "enable"
        ipv6-wins-server1: "<your_own_value>"
        ipv6-wins-server2: "<your_own_value>"
        keep-alive: "enable"
        limit-user-logins: "enable"
        mac-addr-action: "allow"
        mac-addr-check: "enable"
        mac-addr-check-rule:
         -
            mac-addr-list:
             -
                addr: "<your_own_value>"
            mac-addr-mask: "74"
            name: "default_name_75"
        macos-forticlient-download-url: "<your_own_value>"
        name: "default_name_77"
        os-check: "enable"
        os-check-list:
         -
            action: "deny"
            latest-patch-level: "<your_own_value>"
            name: "default_name_82"
            tolerance: "83"
        redir-url: "<your_own_value>"
        save-password: "enable"
        service-restriction: "enable"
        skip-check-for-unsupported-browser: "enable"
        skip-check-for-unsupported-os: "enable"
        smb-ntlmv1-auth: "enable"
        smbv1: "enable"
        split-dns:
         -
            dns-server1: "<your_own_value>"
            dns-server2: "<your_own_value>"
            domains: "<your_own_value>"
            id:  "95"
            ipv6-dns-server1: "<your_own_value>"
            ipv6-dns-server2: "<your_own_value>"
        split-tunneling: "enable"
        split-tunneling-routing-address:
         -
            name: "default_name_100 (source firewall.address.name firewall.addrgrp.name)"
        theme: "blue"
        tunnel-mode: "enable"
        user-bookmark: "enable"
        user-group-bookmark: "enable"
        web-mode: "enable"
        windows-forticlient-download-url: "<your_own_value>"
        wins-server1: "<your_own_value>"
        wins-server2: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_vpn_ssl_web_portal_data(json):
    option_list = ['allow-user-access', 'auto-connect', 'bookmark-group',
                   'custom-lang', 'customize-forticlient-download-url', 'display-bookmark',
                   'display-connection-tools', 'display-history', 'display-status',
                   'dns-server1', 'dns-server2', 'dns-suffix',
                   'exclusive-routing', 'forticlient-download', 'forticlient-download-method',
                   'heading', 'hide-sso-credential', 'host-check',
                   'host-check-interval', 'host-check-policy', 'ip-mode',
                   'ip-pools', 'ipv6-dns-server1', 'ipv6-dns-server2',
                   'ipv6-exclusive-routing', 'ipv6-pools', 'ipv6-service-restriction',
                   'ipv6-split-tunneling', 'ipv6-split-tunneling-routing-address', 'ipv6-tunnel-mode',
                   'ipv6-wins-server1', 'ipv6-wins-server2', 'keep-alive',
                   'limit-user-logins', 'mac-addr-action', 'mac-addr-check',
                   'mac-addr-check-rule', 'macos-forticlient-download-url', 'name',
                   'os-check', 'os-check-list', 'redir-url',
                   'save-password', 'service-restriction', 'skip-check-for-unsupported-browser',
                   'skip-check-for-unsupported-os', 'smb-ntlmv1-auth', 'smbv1',
                   'split-dns', 'split-tunneling', 'split-tunneling-routing-address',
                   'theme', 'tunnel-mode', 'user-bookmark',
                   'user-group-bookmark', 'web-mode', 'windows-forticlient-download-url',
                   'wins-server1', 'wins-server2']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def vpn_ssl_web_portal(data, fos):
    vdom = data['vdom']
    vpn_ssl_web_portal_data = data['vpn_ssl_web_portal']
    flattened_data = flatten_multilists_attributes(vpn_ssl_web_portal_data)
    filtered_data = filter_vpn_ssl_web_portal_data(flattened_data)
    if vpn_ssl_web_portal_data['state'] == "present":
        return fos.set('vpn.ssl.web',
                       'portal',
                       data=filtered_data,
                       vdom=vdom)

    elif vpn_ssl_web_portal_data['state'] == "absent":
        return fos.delete('vpn.ssl.web',
                          'portal',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_vpn_ssl_web(data, fos):
    login(data)

    if data['vpn_ssl_web_portal']:
        resp = vpn_ssl_web_portal(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "vpn_ssl_web_portal": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "allow-user-access": {"required": False, "type": "str",
                                      "choices": ["web", "ftp", "smb",
                                                  "telnet", "ssh", "vnc",
                                                  "rdp", "ping", "citrix",
                                                  "portforward"]},
                "auto-connect": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "bookmark-group": {"required": False, "type": "list",
                                   "options": {
                                       "bookmarks": {"required": False, "type": "list",
                                                     "options": {
                                                         "additional-params": {"required": False, "type": "str"},
                                                         "apptype": {"required": False, "type": "str",
                                                                     "choices": ["citrix", "ftp", "portforward",
                                                                                 "rdp", "smb", "ssh",
                                                                                 "telnet", "vnc", "web"]},
                                                         "description": {"required": False, "type": "str"},
                                                         "folder": {"required": False, "type": "str"},
                                                         "form-data": {"required": False, "type": "list",
                                                                       "options": {
                                                                           "name": {"required": True, "type": "str"},
                                                                           "value": {"required": False, "type": "str"}
                                                                       }},
                                                         "host": {"required": False, "type": "str"},
                                                         "listening-port": {"required": False, "type": "int"},
                                                         "load-balancing-info": {"required": False, "type": "str"},
                                                         "logon-password": {"required": False, "type": "str"},
                                                         "logon-user": {"required": False, "type": "str"},
                                                         "name": {"required": True, "type": "str"},
                                                         "port": {"required": False, "type": "int"},
                                                         "preconnection-blob": {"required": False, "type": "str"},
                                                         "preconnection-id": {"required": False, "type": "int"},
                                                         "remote-port": {"required": False, "type": "int"},
                                                         "security": {"required": False, "type": "str",
                                                                      "choices": ["rdp", "nla", "tls",
                                                                                  "any"]},
                                                         "server-layout": {"required": False, "type": "str",
                                                                           "choices": ["de-de-qwertz", "en-gb-qwerty", "en-us-qwerty",
                                                                                       "es-es-qwerty", "fr-fr-azerty", "fr-ch-qwertz",
                                                                                       "it-it-qwerty", "ja-jp-qwerty", "pt-br-qwerty",
                                                                                       "sv-se-qwerty", "tr-tr-qwerty", "failsafe"]},
                                                         "show-status-window": {"required": False, "type": "str",
                                                                                "choices": ["enable", "disable"]},
                                                         "sso": {"required": False, "type": "str",
                                                                 "choices": ["disable", "static", "auto"]},
                                                         "sso-credential": {"required": False, "type": "str",
                                                                            "choices": ["sslvpn-login", "alternative"]},
                                                         "sso-credential-sent-once": {"required": False, "type": "str",
                                                                                      "choices": ["enable", "disable"]},
                                                         "sso-password": {"required": False, "type": "str"},
                                                         "sso-username": {"required": False, "type": "str"},
                                                         "url": {"required": False, "type": "str"}
                                                     }},
                                       "name": {"required": True, "type": "str"}
                                   }},
                "custom-lang": {"required": False, "type": "str"},
                "customize-forticlient-download-url": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                "display-bookmark": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "display-connection-tools": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "display-history": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "display-status": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "dns-server1": {"required": False, "type": "str"},
                "dns-server2": {"required": False, "type": "str"},
                "dns-suffix": {"required": False, "type": "str"},
                "exclusive-routing": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "forticlient-download": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "forticlient-download-method": {"required": False, "type": "str",
                                                "choices": ["direct", "ssl-vpn"]},
                "heading": {"required": False, "type": "str"},
                "hide-sso-credential": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "host-check": {"required": False, "type": "str",
                               "choices": ["none", "av", "fw",
                                           "av-fw", "custom"]},
                "host-check-interval": {"required": False, "type": "int"},
                "host-check-policy": {"required": False, "type": "list",
                                      "options": {
                                          "name": {"required": True, "type": "str"}
                                      }},
                "ip-mode": {"required": False, "type": "str",
                            "choices": ["range", "user-group"]},
                "ip-pools": {"required": False, "type": "list",
                             "options": {
                                 "name": {"required": True, "type": "str"}
                             }},
                "ipv6-dns-server1": {"required": False, "type": "str"},
                "ipv6-dns-server2": {"required": False, "type": "str"},
                "ipv6-exclusive-routing": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "ipv6-pools": {"required": False, "type": "list",
                               "options": {
                                   "name": {"required": True, "type": "str"}
                               }},
                "ipv6-service-restriction": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "ipv6-split-tunneling": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "ipv6-split-tunneling-routing-address": {"required": False, "type": "list",
                                                         "options": {
                                                             "name": {"required": True, "type": "str"}
                                                         }},
                "ipv6-tunnel-mode": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "ipv6-wins-server1": {"required": False, "type": "str"},
                "ipv6-wins-server2": {"required": False, "type": "str"},
                "keep-alive": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "limit-user-logins": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "mac-addr-action": {"required": False, "type": "str",
                                    "choices": ["allow", "deny"]},
                "mac-addr-check": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "mac-addr-check-rule": {"required": False, "type": "list",
                                        "options": {
                                            "mac-addr-list": {"required": False, "type": "list",
                                                              "options": {
                                                                  "addr": {"required": True, "type": "str"}
                                                              }},
                                            "mac-addr-mask": {"required": False, "type": "int"},
                                            "name": {"required": True, "type": "str"}
                                        }},
                "macos-forticlient-download-url": {"required": False, "type": "str"},
                "name": {"required": True, "type": "str"},
                "os-check": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "os-check-list": {"required": False, "type": "list",
                                  "options": {
                                      "action": {"required": False, "type": "str",
                                                 "choices": ["deny", "allow", "check-up-to-date"]},
                                      "latest-patch-level": {"required": False, "type": "str"},
                                      "name": {"required": True, "type": "str"},
                                      "tolerance": {"required": False, "type": "int"}
                                  }},
                "redir-url": {"required": False, "type": "str"},
                "save-password": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "service-restriction": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "skip-check-for-unsupported-browser": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                "skip-check-for-unsupported-os": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "smb-ntlmv1-auth": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "smbv1": {"required": False, "type": "str",
                          "choices": ["enable", "disable"]},
                "split-dns": {"required": False, "type": "list",
                              "options": {
                                  "dns-server1": {"required": False, "type": "str"},
                                  "dns-server2": {"required": False, "type": "str"},
                                  "domains": {"required": False, "type": "str"},
                                  "id": {"required": True, "type": "int"},
                                  "ipv6-dns-server1": {"required": False, "type": "str"},
                                  "ipv6-dns-server2": {"required": False, "type": "str"}
                              }},
                "split-tunneling": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "split-tunneling-routing-address": {"required": False, "type": "list",
                                                    "options": {
                                                        "name": {"required": True, "type": "str"}
                                                    }},
                "theme": {"required": False, "type": "str",
                          "choices": ["blue", "green", "red",
                                      "melongene", "mariner"]},
                "tunnel-mode": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "user-bookmark": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "user-group-bookmark": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "web-mode": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "windows-forticlient-download-url": {"required": False, "type": "str"},
                "wins-server1": {"required": False, "type": "str"},
                "wins-server2": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_vpn_ssl_web(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()
